@@ -258,6 +258,11 @@ int main() {
 #else
     printf ("(sb-int:style-warn \"Couldn't grovel for ENETUNREACH (unknown to the C compiler).\")\n");
 #endif
+#ifdef ENOTCONN
+    printf ("(cl:defconstant ENOTCONN %d \"NIL\")\n", ENOTCONN);
+#else
+    printf ("(sb-int:style-warn \"Couldn't grovel for ENOTCONN (unknown to the C compiler).\")\n");
+#endif
 #ifdef NETDB_INTERNAL
     printf ("(cl:defconstant NETDB-INTERNAL %d \"See errno.\")\n", NETDB_INTERNAL);
 #else
@@ -452,45 +457,45 @@ int main() {
     printf ("))\n");
     printf ("(cl:declaim (cl:inline SOCKET))\n");
     printf ("(sb-grovel::define-foreign-routine (\"socket\" SOCKET)\n");
-    printf ("  INTEGER\n  (DOMAIN INTEGER)\n  (TYPE INTEGER)\n  (PROTOCOL INTEGER))\n");
+    printf ("  INT\n  (DOMAIN INT)\n  (TYPE INT)\n  (PROTOCOL INT))\n");
     printf ("(cl:declaim (cl:inline BIND))\n");
     printf ("(sb-grovel::define-foreign-routine (\"bind\" BIND)\n");
-    printf ("  INTEGER\n  (SOCKFD INTEGER)\n  (MY-ADDR (* T))\n  (ADDRLEN INTEGER))\n");
+    printf ("  INT\n  (SOCKFD INT)\n  (MY-ADDR (* T))\n  (ADDRLEN INT))\n");
     printf ("(cl:declaim (cl:inline LISTEN))\n");
     printf ("(sb-grovel::define-foreign-routine (\"listen\" LISTEN)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (BACKLOG INTEGER))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (BACKLOG INT))\n");
     printf ("(cl:declaim (cl:inline ACCEPT))\n");
     printf ("(sb-grovel::define-foreign-routine (\"accept\" ACCEPT)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (MY-ADDR (* T))\n  (ADDRLEN INTEGER :IN-OUT))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (MY-ADDR (* T))\n  (ADDRLEN INT :IN-OUT))\n");
     printf ("(cl:declaim (cl:inline GETPEERNAME))\n");
     printf ("(sb-grovel::define-foreign-routine (\"getpeername\" GETPEERNAME)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (HER-ADDR (* T))\n  (ADDRLEN INTEGER :IN-OUT))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (HER-ADDR (* T))\n  (ADDRLEN INT :IN-OUT))\n");
     printf ("(cl:declaim (cl:inline GETSOCKNAME))\n");
     printf ("(sb-grovel::define-foreign-routine (\"getsockname\" GETSOCKNAME)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (MY-ADDR (* T))\n  (ADDRLEN INTEGER :IN-OUT))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (MY-ADDR (* T))\n  (ADDRLEN INT :IN-OUT))\n");
     printf ("(cl:declaim (cl:inline CONNECT))\n");
     printf ("(sb-grovel::define-foreign-routine (\"connect\" CONNECT)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (HIS-ADDR (* T))\n  (ADDRLEN INTEGER))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (HIS-ADDR (* T))\n  (ADDRLEN INT))\n");
     printf ("(cl:declaim (cl:inline CLOSE))\n");
     printf ("(sb-grovel::define-foreign-routine (\"close\" CLOSE)\n");
-    printf ("  INTEGER\n  (FD INTEGER))\n");
+    printf ("  INT\n  (FD INT))\n");
     printf ("(cl:declaim (cl:inline RECVFROM))\n");
     printf ("(sb-grovel::define-foreign-routine (\"recvfrom\" RECVFROM)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (BUF (* T))\n  (LEN INTEGER)\n  (FLAGS INTEGER)\n  (SOCKADDR (* T))\n  (SOCKLEN (* SOCKLEN-T)))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (BUF (* T))\n  (LEN INTEGER)\n  (FLAGS INT)\n  (SOCKADDR (* T))\n  (SOCKLEN (* SOCKLEN-T)))\n");
     printf ("(cl:declaim (cl:inline GETHOSTBYNAME))\n");
     printf ("(sb-grovel::define-foreign-routine (\"gethostbyname\" GETHOSTBYNAME)\n");
     printf ("  (* HOSTENT)\n  (NAME C-STRING))\n");
     printf ("(cl:declaim (cl:inline GETHOSTBYADDR))\n");
     printf ("(sb-grovel::define-foreign-routine (\"gethostbyaddr\" GETHOSTBYADDR)\n");
-    printf ("  (* HOSTENT)\n  (ADDR (* T))\n  (LEN INTEGER)\n  (AF INTEGER))\n");
+    printf ("  (* HOSTENT)\n  (ADDR (* T))\n  (LEN INT)\n  (AF INT))\n");
     printf ("(cl:declaim (cl:inline SETSOCKOPT))\n");
     printf ("(sb-grovel::define-foreign-routine (\"setsockopt\" SETSOCKOPT)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (LEVEL INTEGER)\n  (OPTNAME INTEGER)\n  (OPTVAL (* T))\n  (OPTLEN INTEGER))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (LEVEL INT)\n  (OPTNAME INT)\n  (OPTVAL (* T))\n  (OPTLEN INT))\n");
     printf ("(cl:declaim (cl:inline FCNTL))\n");
     printf ("(sb-grovel::define-foreign-routine (\"fcntl\" FCNTL)\n");
-    printf ("  INTEGER\n  (FD INTEGER)\n  (CMD INTEGER)\n  (ARG INTEGER))\n");
+    printf ("  INT\n  (FD INT)\n  (CMD INT)\n  (ARG LONG))\n");
     printf ("(cl:declaim (cl:inline GETSOCKOPT))\n");
     printf ("(sb-grovel::define-foreign-routine (\"getsockopt\" GETSOCKOPT)\n");
-    printf ("  INTEGER\n  (SOCKET INTEGER)\n  (LEVEL INTEGER)\n  (OPTNAME INTEGER)\n  (OPTVAL (* T))\n  (OPTLEN (* INTEGER)))\n");
+    printf ("  INT\n  (SOCKET INT)\n  (LEVEL INT)\n  (OPTNAME INT)\n  (OPTVAL (* T))\n  (OPTLEN (* INT)))\n");
 return 0;
 }
