@@ -355,11 +355,19 @@
       (sb-posix:syscall-errno c)))
   #.sb-posix::eisdir)
 
+#-(and x86-64 linux)
 (deftest fcntl.1
   (let ((fd (sb-posix:open "/dev/null" sb-posix::o-nonblock)))
     (= (sb-posix:fcntl fd sb-posix::f-getfl) sb-posix::o-nonblock))
   t)
-
+;; On AMD64/Linux O_LARGEFILE is always set, even though the whole
+;; flag makes no sense.
+#+(and x86-64 linux)
+(deftest fcntl.1
+  (let ((fd (sb-posix:open "/dev/null" sb-posix::o-nonblock)))
+    (/= 0 (logand (sb-posix:fcntl fd sb-posix::f-getfl)
+		  sb-posix::o-nonblock)))
+  t)
 
 (deftest opendir.1
   (let ((dir (sb-posix:opendir "/")))
