@@ -15,6 +15,9 @@
 
 echo //entering make-target-2.sh
 
+LANG=C
+export LANG
+
 # Do warm init stuff, e.g. building and loading CLOS, and stuff which
 # can't be done until CLOS is running.
 #
@@ -90,6 +93,7 @@ echo //doing warm init
 	#+sb-show (setf sb-int:*/show* nil)
         ;; The system is complete now, all standard functions are
         ;; defined.
+        (sb-kernel::ctype-of-cache-clear)
         (setq sb-c::*flame-on-necessarily-undefined-function* t)
 	(sb-ext:save-lisp-and-die "output/sbcl.core" :purify t)
 	EOF
