@@ -51,18 +51,16 @@
 # require a second pass, just testing at build-the-cross-compiler time
 # whether the cross-compilation host returns suitable values from 
 # UPGRADED-ARRAY-ELEMENT-TYPE?)
-# FIXME: --noprogrammer was deprecated in sbcl-0.7.5, replaced by 
-# --disable-debugger. We still use the old form here because the
-# change was not preannounced, and it would be rude to make our new
-# version of SBCL unbootstrappable by immediately prior versions.
-# But in a year or so the --noprogrammer here can change to
-# --disable-debugger (and the deprecated --noprogrammer support can
-# go away completely).
+
+LANG=C
+LC_ALL=C
+export LANG
+export LC_ALL
 
 build_started=`date`
 echo "//starting build: $build_started"
 
-SBCL_XC_HOST="${1:-sbcl --noprogrammer}"
+SBCL_XC_HOST="${1:-sbcl --disable-debugger --userinit /dev/null --sysinit /dev/null}"
 export SBCL_XC_HOST
 echo //SBCL_XC_HOST=\"$SBCL_XC_HOST\"
 
@@ -109,15 +107,15 @@ time sh make-target-contrib.sh || exit 1
 
 # Sometimes people used to see the "No tests failed." output from the last
 # DEFTEST in contrib self-tests and think that's all that is. So...
-FLAG=true
+HEADER_HAS_BEEN_PRINTED=false
 for dir in contrib/*
 do
-  if [ -d "$dir" -a -e "$dir/Makefile" -a ! -e "$dir/test-passed" ]; then
-      if $FLAG; then
+  if [ -d "$dir" -a -f "$dir/Makefile" -a ! -f "$dir/test-passed" ]; then
+      if $HEADER_HAS_BEEN_PRINTED; then
 	  echo > /dev/null
       else
 	  echo "Failed contribs:"
-	  FLAG=false
+	  HEADER_HAS_BEEN_PRINTED=true
       fi
       echo "  `basename $dir`"
   fi
@@ -128,11 +126,15 @@ NPASSED=`find contrib -name test-passed -print | wc -l`
 
 echo
 echo "The build seems to have finished successfully, including $NPASSED (out of $NCONTRIBS)"
-echo "contributed modules. If you would like to run more extensive tests (but" 
-echo "expect some failures on non-x86 platforms) on the new SBCL, you can try:"
+echo "contributed modules. If you would like to run more extensive tests on" 
+echo "the new SBCL, you can try:"
 echo
 echo "  cd tests && sh ./run-tests.sh"
 echo
+echo "  (All tests should pass on x86/Linux, x86/FreeBSD4, and ppc/Darwin. On"
+echo "  other platforms some failures are currently expected; patches welcome"
+echo "  as always.)"
+echo
 echo "To build documentation:"
 echo
 echo "  cd doc/manual && make"
