@@ -32,7 +32,7 @@
 			 (%primitive code-instructions code))))
      (ecase kind
        (:jmp-hint
-	(assert (zerop (ldb (byte 2 0) value)))
+	(aver (zerop (ldb (byte 2 0) value)))
 	#+nil
 	(setf (sap-ref-16 sap offset)
 	      (logior (sap-ref-16 sap offset)
@@ -157,9 +157,7 @@
            (vector (make-array length :element-type '(unsigned-byte 8))))
       (declare (type (unsigned-byte 8) length)
                (type (simple-array (unsigned-byte 8) (*)) vector))
-      (copy-from-system-area pc (* n-byte-bits 5)
-                             vector (* n-word-bits vector-data-offset)
-                             (* length n-byte-bits))
+      (copy-ub8-from-system-area pc 5 vector 0 length)
       (let* ((index 0)
              (error-number (sb!c:read-var-integer vector index)))
         (collect ((sc-offsets))
