@@ -48,7 +48,7 @@
 ;;; interpreter stubs for floating point modes get/setters for the
 ;;; alpha have been removed to alpha-vm.lisp, as they are implemented
 ;;; in C rather than as VOPs.
-#!-alpha
+#!-(or alpha x86-64)
 (progn
   (defun floating-point-modes () 
     (floating-point-modes))
@@ -171,7 +171,7 @@
 	   (error 'floating-point-underflow))
 	  ((not (zerop (logand float-inexact-trap-bit traps)))
 	   (error 'floating-point-inexact))
-	  #!+FreeBSD
+	  #!+freebsd
 	  ((zerop (ldb float-exceptions-byte modes))
 	   ;; I can't tell what caused the exception!!
 	   (error 'floating-point-exception
