@@ -169,10 +169,7 @@
     (declare (type system-area-pointer pc)
 	     (type (unsigned-byte 8) length)
 	     (type (simple-array (unsigned-byte 8) (*)) vector))
-    (copy-from-system-area pc (* sb!vm:n-byte-bits 5)
-			   vector (* sb!vm:n-word-bits
-				     sb!vm:vector-data-offset)
-			   (* length sb!vm:n-byte-bits))
+    (copy-ub8-from-system-area pc 5 vector 0 length)
     (let* ((index 0)
 	   (error-number (sb!c:read-var-integer vector index)))
       (collect ((sc-offsets))
