@@ -48,6 +48,7 @@
   (def-type-predicate-wrapper atom)
   (def-type-predicate-wrapper base-char-p)
   (def-type-predicate-wrapper base-string-p)
+  #!+sb-unicode (def-type-predicate-wrapper character-string-p)
   (def-type-predicate-wrapper bignump)
   (def-type-predicate-wrapper bit-vector-p)
   (def-type-predicate-wrapper characterp)
@@ -82,6 +83,7 @@
   (def-type-predicate-wrapper simple-array-p)
   (def-type-predicate-wrapper simple-bit-vector-p)
   (def-type-predicate-wrapper simple-base-string-p)
+  #!+sb-unicode (def-type-predicate-wrapper simple-character-string-p)
   (def-type-predicate-wrapper simple-string-p)
   (def-type-predicate-wrapper simple-vector-p)
   (def-type-predicate-wrapper single-float-p)
@@ -108,7 +110,8 @@
   #!+long-float (def-type-predicate-wrapper simple-array-long-float-p)
   (def-type-predicate-wrapper simple-array-complex-single-float-p)
   (def-type-predicate-wrapper simple-array-complex-double-float-p)
-  #!+long-float (def-type-predicate-wrapper simple-array-complex-long-float-p))
+  #!+long-float (def-type-predicate-wrapper simple-array-complex-long-float-p)
+  (def-type-predicate-wrapper vector-nil-p))
 
 ;;; Return the specifier for the type of object. This is not simply
 ;;; (TYPE-SPECIFIER (CTYPE-OF OBJECT)) because CTYPE-OF has different
