@@ -21,7 +21,12 @@
 (defun sharp-left-paren (stream ignore length)
   (declare (ignore ignore) (special *backquote-count*))
   (let* ((list (read-list stream nil))
-	 (listlength (length list)))
+	 (listlength (handler-case (length list)
+		       (type-error
+			(error)
+			(declare (ignore error))
+			(%reader-error stream "improper list in #(): ~S"
+				       list)))))
     (declare (list list)
 	     (fixnum listlength))
     (cond (*read-suppress* nil)
@@ -159,9 +164,12 @@
                               designator: ~S."
 			     slot-name))
 			  (when (not (keywordp slot-name))
-			    (style-warn "in #S ~S, the use of non-keywords ~
-                                         as slot specifiers is deprecated: ~S."
-					(car body) slot-name)))
+                            (warn 'structure-initarg-not-keyword
+                                  :format-control
+                                  "in #S ~S, the use of non-keywords ~
+                                   as slot specifiers is deprecated: ~S."
+                                  :format-arguments
+                                  (list (car body) slot-name))))
 		     collect (intern (string (car tail)) *keyword-package*)
 		     collect (cadr tail)))))))
 
@@ -169,20 +177,20 @@
 
 (defun sharp-B (stream sub-char numarg)
   (ignore-numarg sub-char numarg)
-  (sharp-r stream sub-char 2))
+  (sharp-R stream sub-char 2))
 
 (defun sharp-C (stream sub-char numarg)
   (ignore-numarg sub-char numarg)
   ;; The next thing had better be a list of two numbers.
   (let ((cnum (read stream t nil t)))
-    (when *read-suppress* (return-from sharp-c nil))
+    (when *read-suppress* (return-from sharp-C nil))
     (if (and (listp cnum) (= (length cnum) 2))
 	(complex (car cnum) (cadr cnum))
 	(%reader-error stream "illegal complex number format: #C~S" cnum))))
 
 (defun sharp-O (stream sub-char numarg)
   (ignore-numarg sub-char numarg)
-  (sharp-r stream sub-char 8))
+  (sharp-R stream sub-char 8))
 
 (defun sharp-R (stream sub-char radix)
   (cond (*read-suppress*
@@ -205,7 +213,7 @@
 
 (defun sharp-X (stream sub-char numarg)
   (ignore-numarg sub-char numarg)
-  (sharp-r stream sub-char 16))
+  (sharp-R stream sub-char 16))
 
 ;;;; reading circular data: the #= and ## readmacros
 
@@ -347,35 +355,41 @@
 
 (defun sharp-vertical-bar (stream sub-char numarg)
   (ignore-numarg sub-char numarg)
-  (let ((stream (in-synonym-of stream)))
-    (if (ansi-stream-p stream)
-	(prepare-for-fast-read-char stream
+  (handler-bind
+      ((character-decoding-error
+	#'(lambda (decoding-error)
+	    (declare (ignorable decoding-error))
+	    (style-warn "Character decoding error in a #|-comment at position ~A reading source file ~A, resyncing." (file-position stream) stream)
+	    (invoke-restart 'attempt-resync))))
+    (let ((stream (in-synonym-of stream)))
+      (if (ansi-stream-p stream)
+	  (prepare-for-fast-read-char stream
+	    (do ((level 1)
+		 (prev (fast-read-char) char)
+		 (char (fast-read-char) (fast-read-char)))
+		(())
+	      (cond ((and (char= prev #\|) (char= char #\#))
+		     (setq level (1- level))
+		     (when (zerop level)
+		       (done-with-fast-read-char)
+		       (return (values)))
+		     (setq char (fast-read-char)))
+		    ((and (char= prev #\#) (char= char #\|))
+		     (setq char (fast-read-char))
+		     (setq level (1+ level))))))
+	  ;; fundamental-stream
 	  (do ((level 1)
-	       (prev (fast-read-char) char)
-	       (char (fast-read-char) (fast-read-char)))
+	       (prev (read-char stream t) char)
+	       (char (read-char stream t) (read-char stream t)))
 	      (())
 	    (cond ((and (char= prev #\|) (char= char #\#))
 		   (setq level (1- level))
 		   (when (zerop level)
-		     (done-with-fast-read-char)
 		     (return (values)))
-		   (setq char (fast-read-char)))
+		   (setq char (read-char stream t)))
 		  ((and (char= prev #\#) (char= char #\|))
-		   (setq char (fast-read-char))
-		   (setq level (1+ level))))))
-	;; fundamental-stream
-	(do ((level 1)
-	     (prev (read-char stream t) char)
-	     (char (read-char stream t) (read-char stream t)))
-	    (())
-	  (cond ((and (char= prev #\|) (char= char #\#))
-		 (setq level (1- level))
-		 (when (zerop level)
-		   (return (values)))
-		 (setq char (read-char stream t)))
-		((and (char= prev #\#) (char= char #\|))
-		 (setq char (read-char stream t))
-		 (setq level (1+ level))))))))
+		   (setq char (read-char stream t))
+		   (setq level (1+ level)))))))))
 
 ;;;; a grab bag of other sharp readmacros: #', #:, and #.
 
@@ -441,8 +455,8 @@
   (set-dispatch-macro-character #\# #\C #'sharp-C)
   (set-dispatch-macro-character #\# #\c #'sharp-C)
   (set-dispatch-macro-character #\# #\| #'sharp-vertical-bar)
-  (set-dispatch-macro-character #\# #\p #'sharp-p)
-  (set-dispatch-macro-character #\# #\P #'sharp-p)
+  (set-dispatch-macro-character #\# #\p #'sharp-P)
+  (set-dispatch-macro-character #\# #\P #'sharp-P)
   (set-dispatch-macro-character #\# #\) #'sharp-illegal)
   (set-dispatch-macro-character #\# #\< #'sharp-illegal)
   (set-dispatch-macro-character #\# #\Space #'sharp-illegal)
