@@ -33,6 +33,7 @@
   `(integer 0 (,char-code-limit)))
 
 (defvar *character-database*)
+(declaim (type (simple-array (unsigned-byte 8) (*)) *character-database*))
 
 (macrolet ((frob ()
              (with-open-file (stream (merge-pathnames
@@ -41,7 +42,7 @@
                                        '(:relative :up :up "output")
                                        :name "ucd"
                                        :type "dat")
-                                      sb!xc:*compile-file-pathname*)
+                                      sb!xc:*compile-file-truename*)
                                      :direction :input
                                      :element-type '(unsigned-byte 8))
                (let* ((length (file-length stream))
