@@ -237,9 +237,11 @@ error if any of PACKAGES is not a valid package designator."
   #!+sb-package-locks
   (let* ((symbol (etypecase name
 		   (symbol name)
-		   (list (if (eq 'setf (first name))
+		   (list (if (and (consp (cdr name))
+				  (eq 'setf (first name)))
 			     (second name)
-			     ;; Skip (class-predicate foo), etc.
+			     ;; Skip lists of length 1, single conses and
+			     ;; (class-predicate foo), etc.
 			     ;; FIXME: MOP and package-lock
 			     ;; interaction needs to be thought about.
 			     (return-from 
@@ -477,15 +479,17 @@ error if any of PACKAGES is not a valid package designator."
 	     (push n (package-%nicknames package)))
 	    ((eq found package))
 	    ((string= (the string (package-%name found)) n)
-             (error 'simple-package-error
-                    :package package
-                    :format-control "~S is a package name, so it cannot be a nickname for ~S."
-                    :format-arguments (list n (package-%name package))))
+             (cerror "Ignore this nickname."
+		     'simple-package-error
+		     :package package
+		     :format-control "~S is a package name, so it cannot be a nickname for ~S."
+		     :format-arguments (list n (package-%name package))))
 	    (t
-             (error 'simple-package-error
-                    :package package
-                    :format-control "~S is already a nickname for ~S."
-                    :format-arguments (list n (package-%name found))))))))
+             (cerror "Leave this nickname alone."
+		     'simple-package-error
+		     :package package
+		     :format-control "~S is already a nickname for ~S."
+		     :format-arguments (list n (package-%name found))))))))
 
 (defun make-package (name &key
 			  (use '#.*default-package-use-list*)
@@ -647,8 +651,8 @@ error if any of PACKAGES is not a valid package designator."
   ;; We just simple-stringify the name and call INTERN*, where the real
   ;; logic is.
   (let ((name (if (simple-string-p name)
-		name
-		(coerce name 'simple-string)))
+		  name
+		  (coerce name 'simple-string)))
 	(package (find-undeleted-package-or-lose package)))
     (declare (simple-string name))
       (intern* name
