@@ -19,10 +19,6 @@
     unsigned-long
   (lisp-fun-address unsigned-long))
 
-(define-alien-routine "signal_thread_to_dequeue"
-    unsigned-int
-  (thread-id unsigned-long))
-
 (define-alien-routine reap-dead-threads void)
 
 (defvar *session* nil)
@@ -34,14 +30,18 @@
 
 (defun get-spinlock (lock offset new-value)
   (declare (optimize (speed 3) (safety 0)))
+  ;; %instance-set-conditional can test for 0 (which is a fixnum) and
+  ;; store any value
   (loop until
-	(eql (sb!vm::%instance-set-conditional lock offset 0 new-value) 0)))
+       (eql (sb!vm::%instance-set-conditional lock offset 0 new-value) 0)))
 
-;; this should do nothing if we didn't own the lock, so safe to use in
-;; unwind-protect cleanups when lock acquisition failed for some reason
-(defun release-spinlock (lock offset our-value)
+(defun release-spinlock (lock offset)
   (declare (optimize (speed 3) (safety 0)))
-  (sb!vm::%instance-set-conditional lock offset our-value 0))
+  ;; %instance-set-conditional cannot compare arbitrary objects
+  ;; meaningfully, so 
+  ;; (sb!vm::%instance-set-conditional lock offset our-value 0)
+  ;; does not work for bignum thread ids.
+  (sb!vm::%instance-set lock offset 0))
 
 (defmacro with-spinlock ((queue) &body body)
   (with-unique-names (pid)
@@ -50,7 +50,7 @@
 	    (progn
 	      (get-spinlock ,queue 2 ,pid)
 	      ,@body)
-	 (release-spinlock ,queue 2 ,pid)))))
+	 (release-spinlock ,queue 2)))))
 
 
 ;;;; the higher-level locking operations are based on waitqueues
@@ -81,70 +81,19 @@
    (+ (sb!kernel:get-lisp-obj-address lock)
       (- (* 5 sb!vm:n-word-bytes) sb!vm:instance-pointer-lowtag))))
 
-(sb!alien:define-alien-routine "block_sigcont"  void)
-(sb!alien:define-alien-routine "unblock_sigcont_and_sleep"  void)
-
-#!+sb-futex
 (declaim (inline futex-wait futex-wake))
-#!+sb-futex
 (sb!alien:define-alien-routine
     "futex_wait" int (word unsigned-long) (old-value unsigned-long))
-#!+sb-futex
 (sb!alien:define-alien-routine
     "futex_wake" int (word unsigned-long) (n unsigned-long))
 
 
-;;; this should only be called while holding the queue spinlock.
-;;; it releases the spinlock before sleeping
-(defun wait-on-queue (queue &optional lock)
-  (let ((pid (current-thread-id)))
-    (block-sigcont)
-    (when lock (release-mutex lock))
-    (sb!sys:without-interrupts
-     (pushnew pid (waitqueue-data queue)))
-    (setf (waitqueue-lock queue) 0)
-    (unblock-sigcont-and-sleep)))
-
-;;; this should only be called while holding the queue spinlock.  It doesn't
-;;; release it
-(defun dequeue (queue)
-  (let ((pid (current-thread-id)))
-    (sb!sys:without-interrupts     
-     (setf (waitqueue-data queue)
-	   (delete pid (waitqueue-data queue))))))
-
-;;; this should only be called while holding the queue spinlock.
-(defun signal-queue-head (queue)
-  (let ((p (car (waitqueue-data queue))))
-    (when p (signal-thread-to-dequeue p))))
-
 ;;;; mutex
 
-;;; i suspect there may be a race still in this: the futex version requires
-;;; the old mutex value before sleeping, so how do we get away without it
 (defun get-mutex (lock &optional new-value (wait-p t))
   "Acquire LOCK, setting it to NEW-VALUE or some suitable default value 
 if NIL.  If WAIT-P is non-NIL and the lock is in use, sleep until it
 is available"
-  (declare (type mutex lock) (optimize (speed 3)))
-  (let ((pid (current-thread-id)))
-    (unless new-value (setf new-value pid))
-    (assert (not (eql new-value (mutex-value lock))))
-    (get-spinlock lock 2 pid)
-    (loop
-     (unless
-	 ;; args are object slot-num old-value new-value
-	 (sb!vm::%instance-set-conditional lock 4 nil new-value)
-       (dequeue lock)
-       (setf (waitqueue-lock lock) 0)
-       (return t))
-     (unless wait-p
-       (setf (waitqueue-lock lock) 0)
-       (return nil))
-     (wait-on-queue lock nil))))
-
-#!+sb-futex
-(defun get-mutex/futex (lock &optional new-value (wait-p t))
   (declare (type mutex lock)  (optimize (speed 3)))
   (let ((pid (current-thread-id))
 	old)
@@ -159,15 +108,7 @@ is available"
      (futex-wait (mutex-value-address lock)
 		 (sb!kernel:get-lisp-obj-address old)))))
 
-(defun release-mutex (lock &optional (new-value nil))
-  (declare (type mutex lock))
-  ;; we assume the lock is ours to release
-  (with-spinlock (lock)
-    (setf (mutex-value lock) new-value)
-    (signal-queue-head lock)))
-
-#!+sb-futex
-(defun release-mutex/futex (lock)
+(defun release-mutex (lock)
   (declare (type mutex lock))
   (setf (mutex-value lock) nil)
   (futex-wake (mutex-value-address lock) 1))
@@ -178,22 +119,6 @@ is available"
   "Atomically release LOCK and enqueue ourselves on QUEUE.  Another
 thread may subsequently notify us using CONDITION-NOTIFY, at which
 time we reacquire LOCK and return to the caller."
-  (assert lock)
-  (let ((value (mutex-value lock)))
-    (unwind-protect
-	 (progn
-	   (get-spinlock queue 2 (current-thread-id))
-	   (wait-on-queue queue lock))
-      ;; If we are interrupted while waiting, we should do these things
-      ;; before returning.  Ideally, in the case of an unhandled signal,
-      ;; we should do them before entering the debugger, but this is
-      ;; better than nothing.
-      (with-spinlock (queue)
-	(dequeue queue))
-      (get-mutex lock value))))
-
-#!+sb-futex
-(defun condition-wait/futex (queue lock)
   (assert lock)
   (let ((value (mutex-value lock)))
     (unwind-protect
@@ -218,11 +143,6 @@ time we reacquire LOCK and return to the caller."
 
 (defun condition-notify (queue)
   "Notify one of the processes waiting on QUEUE"
-  (with-spinlock (queue) (signal-queue-head queue)))
-
-#!+sb-futex
-(defun condition-notify/futex (queue)
-  "Notify one of the processes waiting on QUEUE."
   (let ((me (current-thread-id)))
     ;; no problem if >1 thread notifies during the comment in
     ;; condition-wait: as long as the value in queue-data isn't the
@@ -232,30 +152,11 @@ time we reacquire LOCK and return to the caller."
     (setf (waitqueue-data queue) me)
     (futex-wake (waitqueue-data-address queue) 1)))
 
-#!+sb-futex
-(defun condition-broadcast/futex (queue)
+(defun condition-broadcast (queue)
   (let ((me (current-thread-id)))
     (setf (waitqueue-data queue) me)
     (futex-wake (waitqueue-data-address queue) (ash 1 30))))
 
-(defun condition-broadcast (queue)
-  "Notify all of the processes waiting on QUEUE."
-  (with-spinlock (queue)
-    (map nil #'signal-thread-to-dequeue (waitqueue-data queue))))
-
-;;; Futexes may be available at compile time but not runtime, so we
-;;; default to not using them unless os_init says they're available
-(defun maybe-install-futex-functions ()
-  #!+sb-futex
-  (unless (zerop (extern-alien "linux_supports_futex" int))
-    (sb!ext:without-package-locks
-      (setf (fdefinition 'get-mutex) #'get-mutex/futex
-            (fdefinition 'release-mutex) #'release-mutex/futex
-            (fdefinition 'condition-wait) #'condition-wait/futex
-            (fdefinition 'condition-broadcast) #'condition-broadcast/futex
-            (fdefinition 'condition-notify) #'condition-notify/futex))
-    t))
-
 (defun make-thread (function)
   (let* ((real-function (coerce function 'function))
 	 (tid
@@ -272,35 +173,26 @@ time we reacquire LOCK and return to the caller."
 		;; can't use handling-end-of-the-world, because that flushes
 		;; output streams, and we don't necessarily have any (or we
 		;; could be sharing them)
-		(sb!sys:enable-interrupt sb!unix:sigint :ignore)
 		(catch 'sb!impl::%end-of-the-world 
 		  (with-simple-restart 
-		      (destroy-thread
-		       (format nil "~~@<Destroy this thread (~A)~~@:>"
+		      (terminate-thread
+		       (format nil "~~@<Terminate this thread (~A)~~@:>"
 			       (current-thread-id)))
+                    ;; now that most things have a chance to work
+                    ;; properly without messing up other threads, it's
+                    ;; time to enable signals
+                    (sb!unix::reset-signal-mask)
 		    (funcall real-function))
 		  0))
 	      (values))))))
+    (when (zerop tid) (error "Can't create a new thread"))
     (with-mutex ((session-lock *session*))
       (pushnew tid (session-threads *session*)))
     tid))
 
-;;; Really, you don't want to use these: they'll get into trouble with
-;;; garbage collection.  Use a lock or a waitqueue instead
-(defun suspend-thread (thread-id)
-  (sb!unix:unix-kill thread-id sb!unix:sigstop))
-(defun resume-thread (thread-id)
-  (sb!unix:unix-kill thread-id sb!unix:sigcont))
-;;; Note warning about cleanup forms
 (defun destroy-thread (thread-id)
-  "Destroy the thread identified by THREAD-ID abruptly, without running cleanup forms"
-  (sb!unix:unix-kill thread-id sb!unix:sigterm)
-  ;; may have been stopped for some reason, so now wake it up to
-  ;; deliver the TERM
-  (sb!unix:unix-kill thread-id sb!unix:sigcont))
-
-     
-     
+  "Deprecated. Soon to be removed or reimplemented using pthread_cancel."
+  (terminate-thread thread-id))
 
 ;;; a moderate degree of care is expected for use of interrupt-thread,
 ;;; due to its nature: if you interrupt a thread that was holding
@@ -320,16 +212,14 @@ time we reacquire LOCK and return to the caller."
 (defun interrupt-thread (thread function)
   "Interrupt THREAD and make it run FUNCTION."
   (let ((function (coerce function 'function)))
-    (sb!sys:with-pinned-objects 
-     (function)
-     (multiple-value-bind (res err)
-	 (sb!unix::syscall ("interrupt_thread"
-			    sb!alien:unsigned-long  sb!alien:unsigned-long)
-			   thread
-			   thread 
-			   (sb!kernel:get-lisp-obj-address function))
-       (unless res
-	 (error 'interrupt-thread-error :thread thread :errno err))))))
+    (multiple-value-bind (res err)
+        (sb!unix::syscall ("interrupt_thread"
+                           sb!alien:unsigned-long  sb!alien:unsigned-long)
+                          thread
+                          thread 
+                          (sb!kernel:get-lisp-obj-address function))
+      (unless res
+        (error 'interrupt-thread-error :thread thread :errno err)))))
 
 
 (defun terminate-thread (thread-id)
@@ -339,11 +229,8 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
 
 (declaim (inline current-thread-id))
 (defun current-thread-id ()
-  (logand 
-   (sb!sys:sap-int
-    (sb!vm::current-thread-offset-sap sb!vm::thread-pid-slot))
-   ;; KLUDGE pids are 16 bit really.  Avoid boxing the return value
-   (1- (ash 1 16))))
+  (sb!sys:sap-int
+   (sb!vm::current-thread-offset-sap sb!vm::thread-os-thread-slot)))
 
 ;;;; iterate over the in-memory threads
 
@@ -351,7 +238,8 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
   "Call FUNCTION once for each known thread, giving it the thread structure as argument"
   (let ((function (coerce function 'function)))
     (loop for thread = (alien-sap (extern-alien "all_threads" (* t)))
-	  then  (sb!sys:sap-ref-sap thread (* 4 sb!vm::thread-next-slot))
+	  then  (sb!sys:sap-ref-sap thread (* sb!vm:n-word-bytes
+					      sb!vm::thread-next-slot))
 	  until (sb!sys:sap= thread (sb!sys:int-sap 0))
 	  collect (funcall function thread))))
 
@@ -359,9 +247,12 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
   (let ((thread (alien-sap (extern-alien "all_threads" (* t)))))
     (loop 
      (when (sb!sys:sap= thread (sb!sys:int-sap 0)) (return nil))
-     (let ((pid (sb!sys:sap-ref-32 thread (* 4 sb!vm::thread-pid-slot))))
+     ;; FIXME: 32/64 bit
+     (let ((pid (sb!sys:sap-ref-32 thread (* sb!vm:n-word-bytes
+					     sb!vm::thread-os-thread-slot))))
        (when (= pid id) (return thread))
-       (setf thread (sb!sys:sap-ref-sap thread (* 4 sb!vm::thread-next-slot)))))))
+       (setf thread (sb!sys:sap-ref-sap thread (* sb!vm:n-word-bytes
+						  sb!vm::thread-next-slot)))))))
 
 ;;; internal use only.  If you think you need to use this, either you
 ;;; are an SBCL developer, are doing something that you should discuss
@@ -371,7 +262,8 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
   (let ((thread (thread-sap-from-id thread-id)))
     (when thread
       (let* ((index (sb!vm::symbol-tls-index symbol))
-	     (tl-val (sb!sys:sap-ref-32 thread (* 4 index))))
+	     (tl-val (sb!sys:sap-ref-word thread
+					  (* sb!vm:n-word-bytes index))))
 	(if (eql tl-val sb!vm::unbound-marker-widetag)
 	    (sb!vm::symbol-global-value symbol)
 	    (sb!kernel:make-lisp-obj tl-val))))))
@@ -379,7 +271,7 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
 ;;;; job control, independent listeners
 
 (defstruct session 
-  (lock (make-mutex))
+  (lock (make-mutex :name "session lock"))
   (threads nil)
   (interactive-threads nil)
   (interactive-threads-queue (make-waitqueue)))
@@ -410,17 +302,16 @@ SB-EXT:QUIT - the usual cleanup forms will be evaluated"
       (call-with-new-session (function ,fb-name)))))
 
 ;;; Remove thread id TID from its session, if it has one.  This is
-;;; called from C reap_dead_threads() so is run in the context of
-;;; whichever thread called that (usually after a GC), which may not have 
-;;; any meaningful parent/child/sibling relationship with the dead thread
+;;; called from C mark_thread_dead().
 (defun handle-thread-exit (tid)
-  (let ((session (symbol-value-in-thread '*session* tid)))
-    (and session (%delete-thread-from-session tid session))))
-  
+  (when *session*
+    (%delete-thread-from-session tid *session*)))
+
 (defun terminate-session ()
   "Kill all threads in session except for this one.  Does nothing if current
 thread is not the foreground thread"
   (reap-dead-threads)
+  ;; FIXME: threads created in other threads may escape termination
   (let* ((tid (current-thread-id))
 	 (to-kill
 	  (with-mutex ((session-lock *session*))
@@ -429,7 +320,11 @@ thread is not the foreground thread"
     ;; do the kill after dropping the mutex; unwind forms in dying
     ;; threads may want to do session things
     (dolist (p to-kill)
-      (unless (eql p tid) (terminate-thread p)))))
+      (unless (eql p tid)
+        ;; terminate the thread but don't be surprised if it has
+        ;; exited in the meantime
+        (handler-case (terminate-thread p)
+          (interrupt-thread-error ()))))))
 
 ;;; called from top of invoke-debugger
 (defun debugger-wait-until-foreground-thread (stream)
@@ -452,7 +347,6 @@ interactive."
 	 (when (eql (car int-t) tid)
 	   (unless was-foreground
 	     (format *query-io* "Resuming thread ~A~%" tid))
-	   (sb!sys:enable-interrupt sb!unix:sigint #'sb!unix::sigint-handler)
 	   (return-from get-foreground t))
 	 (setf was-foreground nil)
 	 (unless (member tid int-t)
@@ -468,13 +362,15 @@ interactive."
     (let ((tid (current-thread-id)))
       (setf (session-interactive-threads *session*)
 	    (delete tid (session-interactive-threads *session*)))
-      (sb!sys:enable-interrupt sb!unix:sigint :ignore)
       (when next 
 	(setf (session-interactive-threads *session*)
 	      (list* next 
 		     (delete next (session-interactive-threads *session*)))))
       (condition-broadcast (session-interactive-threads-queue *session*)))))
 
+(defun foreground-thread ()
+  (car (session-interactive-threads *session*)))
+
 (defun make-listener-thread (tty-name)  
   (assert (probe-file tty-name))
   (let* ((in (sb!unix:unix-open (namestring tty-name) sb!unix:o_rdwr #o666))
@@ -483,16 +379,15 @@ interactive."
     (labels ((thread-repl () 
 	       (sb!unix::unix-setsid)
 	       (let* ((sb!impl::*stdin* 
-		       (sb!sys:make-fd-stream in :input t :buffering :line))
+		       (sb!sys:make-fd-stream in :input t :buffering :line :dual-channel-p t))
 		      (sb!impl::*stdout* 
-		       (sb!sys:make-fd-stream out :output t :buffering :line))
+		       (sb!sys:make-fd-stream out :output t :buffering :line :dual-channel-p t))
 		      (sb!impl::*stderr* 
-		       (sb!sys:make-fd-stream err :output t :buffering :line))
+		       (sb!sys:make-fd-stream err :output t :buffering :line :dual-channel-p t))
 		      (sb!impl::*tty* 
-		       (sb!sys:make-fd-stream err :input t :output t :buffering :line))
+		       (sb!sys:make-fd-stream err :input t :output t :buffering :line :dual-channel-p t))
 		      (sb!impl::*descriptor-handlers* nil))
 		 (with-new-session ()
-		   (sb!sys:enable-interrupt sb!unix:sigint #'sb!unix::sigint-handler)
 		   (unwind-protect
 			(sb!impl::toplevel-repl nil)
 		     (sb!int:flush-standard-output-streams))))))
