@@ -301,9 +301,7 @@
       (/show0 "LENGTH,VECTOR,ERROR-NUMBER=..")
       (/hexstr length)
       (/hexstr vector)
-      (copy-from-system-area pc (* n-byte-bits 2)
-			     vector (* n-word-bits vector-data-offset)
-			     (* length n-byte-bits))
+      (copy-ub8-from-system-area pc 2 vector 0 length)
       (let* ((index 0)
 	     (error-number (sb!c:read-var-integer vector index)))
 	(/hexstr error-number)
