@@ -252,7 +252,10 @@
                                   (unless (and (sc-is value immediate)
                                                (= (tn-value value)
 						  ,(1- (ash 1 bits))))
-                                    (cond ((= extra ,(1- elements-per-word))
+                                    (cond #+#.(cl:if
+                                             (cl:= sb-vm:n-word-bits sb-vm:n-machine-word-bits)
+                                             '(and) '(or))
+                                          ((= extra ,(1- elements-per-word))
                                            (inst sll old ,bits old)
                                            (inst srl old ,bits old))
                                           (t
@@ -567,7 +570,11 @@
 (define-full-reffer raw-bits * 0 other-pointer-lowtag (unsigned-reg) unsigned-num
   %raw-bits)
 (define-full-setter set-raw-bits * 0 other-pointer-lowtag (unsigned-reg)
-  unsigned-num %set-raw-bits #+gengc nil)
+  unsigned-num %set-raw-bits)
+(define-full-reffer vector-raw-bits * vector-data-offset other-pointer-lowtag
+  (unsigned-reg) unsigned-num %vector-raw-bits)
+(define-full-setter set-vector-raw-bits * vector-data-offset other-pointer-lowtag
+  (unsigned-reg) unsigned-num %set-vector-raw-bits)
 
 
 ;;;; misc. array VOPs
