@@ -87,8 +87,8 @@
 (defmacro lisp-jump (function lip)
   "Jump to the lisp function FUNCTION.  LIP is an interior-reg temporary."
   `(progn
-     (inst lda ,lip (- (ash sb!vm:simple-fun-code-offset sb!vm:word-shift)
-		       sb!vm:fun-pointer-lowtag)
+     (inst lda ,lip (- (ash simple-fun-code-offset word-shift)
+		       fun-pointer-lowtag)
 	    ,function)
      (move ,function code-tn)
      (inst jsr zero-tn ,lip 1)))
@@ -169,28 +169,25 @@
 ;;; presumably initializes the object.
 (defmacro with-fixed-allocation ((result-tn temp-tn widetag size)
 				 &body body)
-  `(pseudo-atomic (:extra (pad-data-block ,size))
-     (inst bis alloc-tn other-pointer-lowtag ,result-tn)
-     (inst li (logior (ash (1- ,size) n-widetag-bits) ,widetag) ,temp-tn)
-     (storew ,temp-tn ,result-tn 0 other-pointer-lowtag)
-     ,@body))
+  (unless body
+    (bug "empty &body in WITH-FIXED-ALLOCATION"))
+  (once-only ((result-tn result-tn) (temp-tn temp-tn) (size size))
+    `(pseudo-atomic (:extra (pad-data-block ,size))
+       (inst bis alloc-tn other-pointer-lowtag ,result-tn)
+       (inst li (logior (ash (1- ,size) n-widetag-bits) ,widetag) ,temp-tn)
+       (storew ,temp-tn ,result-tn 0 other-pointer-lowtag)
+       ,@body)))
+
+(defun align-csp (temp)
+  ;; is used for stack allocation of dynamic-extent objects
+  (let ((aligned (gen-label)))
+    (inst and csp-tn lowtag-mask temp)
+    (inst beq temp aligned)
+    (inst addq csp-tn n-word-bytes csp-tn)
+    (storew zero-tn csp-tn -1)
+    (emit-label aligned)))
 
 ;;;; error code
-
-(defvar *adjustable-vectors* nil)
-
-(defmacro with-adjustable-vector ((var) &rest body)
-  `(let ((,var (or (pop *adjustable-vectors*)
-		   (make-array 16
-			       :element-type '(unsigned-byte 8)
-			       :fill-pointer 0
-			       :adjustable t))))
-     (setf (fill-pointer ,var) 0)
-     (unwind-protect
-	 (progn
-	   ,@body)
-       (push ,var *adjustable-vectors*))))
-
 (eval-when (:compile-toplevel :load-toplevel :execute) 
   (defun emit-error-break (vop kind code values)
     (let ((vector (gensym)))
@@ -239,8 +236,7 @@
   Emit code for a continuable error with the specified Error-Code and
   context Values.  If the error is continued, execution resumes after
   the GENERATE-CERROR-CODE form."
-  (let ((continue (gensym "CONTINUE-LABEL-"))
-	(error (gensym "ERROR-LABEL-")))
+  (with-unique-names (continue error)
     `(let ((,continue (gen-label)))
        (emit-label ,continue)
        (assemble (*elsewhere*)
@@ -260,16 +256,6 @@
 
 ;;;; memory accessor vop generators
 
-(deftype load/store-index (scale lowtag min-offset
-				 &optional (max-offset min-offset))
-  `(integer ,(- (truncate (+ (ash 1 16)
-			     (* min-offset n-word-bytes)
-			     (- lowtag))
-			  scale))
-	    ,(truncate (- (+ (1- (ash 1 16)) lowtag)
-			  (* max-offset n-word-bytes))
-		       scale)))
-
 (defmacro define-full-reffer (name type offset lowtag scs el-type
 				   &optional translate)
   `(progn
@@ -405,7 +391,7 @@
 	 (:result-types ,el-type)
 	 (:temporary (:sc non-descriptor-reg) temp)
 	 (:temporary (:sc non-descriptor-reg) temp1)
-	 (:generator 5
+	 (:generator 4
 	   ,@(ecase size
 	       (:byte
 		(if signed
@@ -498,30 +484,45 @@
 	 (:temporary (:sc non-descriptor-reg) temp2)
 	 (:results (result :scs ,scs))
 	 (:result-types ,el-type)
-	 (:generator 5
+	 (:generator 4
 	   ,@(ecase size
 	       (:byte
-		`((inst lda temp (- (* ,offset n-word-bytes)
-				    (* index ,scale) ,lowtag)
+		`((inst lda temp (- (+ (* ,offset n-word-bytes)
+				       (* index ,scale))
+				    ,lowtag)
 			object)
-		  (inst ldq_u temp1 (- (* ,offset n-word-bytes) 
-				       (* index ,scale) ,lowtag)
+		  (inst ldq_u temp1 (- (+ (* ,offset n-word-bytes) 
+					  (* index ,scale))
+				       ,lowtag)
 			object)
 		  (inst insbl value temp temp2)
 		  (inst mskbl temp1 temp temp1)
 		  (inst bis temp1 temp2 temp1)
-		  (inst stq_u temp1 (- (* ,offset n-word-bytes)
-				       (* index ,scale) ,lowtag) object)))
+		  (inst stq_u temp1 (- (+ (* ,offset n-word-bytes)
+					  (* index ,scale))
+				       ,lowtag) object)))
 	       (:short
-		`((inst lda temp (- (* ,offset n-word-bytes)
-				    (* index ,scale) ,lowtag)
+		`((inst lda temp (- (+ (* ,offset n-word-bytes)
+				       (* index ,scale))
+				    ,lowtag)
 			object)
-		  (inst ldq_u temp1 (- (* ,offset n-word-bytes)
-				       (* index ,scale) ,lowtag)
+		  (inst ldq_u temp1 (- (+ (* ,offset n-word-bytes)
+					  (* index ,scale))
+				       ,lowtag)
 			object)
 		  (inst mskwl temp1 temp temp1)
 		  (inst inswl value temp temp2)
 		  (inst bis temp1 temp2 temp)
-		  (inst stq_u temp (- (* ,offset n-word-bytes)
-				      (* index ,scale) ,lowtag) object))))
+		  (inst stq_u temp (- (+ (* ,offset n-word-bytes)
+					 (* index ,scale))
+				      ,lowtag) object))))
 	   (move value result))))))
+
+(defmacro sb!sys::with-pinned-objects ((&rest objects) &body body)
+  "Arrange with the garbage collector that the pages occupied by
+OBJECTS will not be moved in memory for the duration of BODY.
+Useful for e.g. foreign calls where another thread may trigger
+garbage collection.  This is currently implemented by disabling GC"
+  (declare (ignore objects))		;should we eval these for side-effect?
+  `(without-gcing
+    ,@body))
