@@ -68,7 +68,8 @@
 		      (if (lambda-var-indirect thing)
 			  *backend-t-primitive-type*
 			  (primitive-type (leaf-type thing))))
-		     (nlx-info *backend-t-primitive-type*))))
+		     (nlx-info *backend-t-primitive-type*)
+                     (clambda *backend-t-primitive-type*))))
 	(push (cons thing (make-normal-tn ptype))
 	      reversed-ir2-physenv-alist)))
 
@@ -209,6 +210,8 @@
 	    (make-ir2-nlx-info
 	     :home (when (member (cleanup-kind (nlx-info-cleanup nlx))
 				 '(:block :tagbody))
-		     (make-normal-tn *backend-t-primitive-type*))
+                     (if (nlx-info-safe-p nlx)
+                         (make-normal-tn *backend-t-primitive-type*)
+                         (make-stack-pointer-tn)))
 	     :save-sp (make-nlx-sp-tn physenv)))))
   (values))
