@@ -319,7 +319,7 @@ default-value-8
 	(inst compute-code-from-lra code-tn lra-label temp code-tn)
 	
 	(let ((defaults (defaults)))
-	  (assert defaults)
+	  (aver defaults)
 	  (assemble (*elsewhere*)
 	    (trace-table-entry trace-table-call-site)
 	    DEFAULT-STACK-VALS
@@ -604,7 +604,7 @@ default-value-8
 ;;; the current frame.
 ;;;
 (macrolet ((define-full-call (name named return variable)
-  (assert (not (and variable (eq return :tail))))
+  (aver (not (and variable (eq return :tail))))
   `(define-vop (,name
 		,@(when (eq return :unknown)
 		    '(unknown-values-receiver)))
