@@ -68,9 +68,6 @@
 (def!constant static-space-start    #x28000000)
 (def!constant static-space-end      #x2a000000)
 
-(def!constant dynamic-space-start   #x30000000)
-(def!constant dynamic-space-end     #x37fff000)
-
 (def!constant dynamic-0-space-start   #x30000000)
 (def!constant dynamic-0-space-end     #x37fff000)
 (def!constant dynamic-1-space-start   #x38000000)
@@ -124,7 +121,8 @@
     sb!impl::sub-gc
     sb!kernel::internal-error
     sb!kernel::control-stack-exhausted-error
-    sb!kernel::undefined-alien-error
+    sb!kernel::undefined-alien-variable-error
+    sb!kernel::undefined-alien-function-error
     sb!di::handle-breakpoint
     sb!impl::fdefinition-object
 
