@@ -342,7 +342,7 @@
             (fail "Attempt to bind a ~(~A~) variable with SYMBOL-MACROLET: ~S"
                   kind name)))
 	;; A magical cons that MACROEXPAND-1 understands.
-        `(,name . (MACRO . ,expansion))))))
+        `(,name . (macro . ,expansion))))))
 
 (defun funcall-in-symbol-macrolet-lexenv (definitions fun context)
   (%funcall-in-foomacrolet-lexenv
@@ -435,28 +435,59 @@
   (reference-constant start next result thing))
 
 ;;;; FUNCTION and NAMED-LAMBDA
+(defun name-lambdalike (thing)
+  (ecase (car thing)
+    ((named-lambda)
+     (second thing))
+    ((lambda instance-lambda)
+     `(lambda ,(second thing)))
+    ((lambda-with-lexenv)'
+     `(lambda ,(fifth thing)))))
+
 (defun fun-name-leaf (thing)
   (if (consp thing)
       (cond
 	((member (car thing)
 		 '(lambda named-lambda instance-lambda lambda-with-lexenv))
-	 (ir1-convert-lambdalike
-			  thing
-			  :debug-name (debug-namify "#'" thing)))
+	 (values (ir1-convert-lambdalike
+                  thing
+                  :debug-name (name-lambdalike thing))
+                 t))
 	((legal-fun-name-p thing)
-	 (find-lexically-apparent-fun
-		     thing "as the argument to FUNCTION"))
+	 (values (find-lexically-apparent-fun
+                  thing "as the argument to FUNCTION")
+                 nil))
 	(t
 	 (compiler-error "~S is not a legal function name." thing)))
-      (find-lexically-apparent-fun
-       thing "as the argument to FUNCTION")))
+      (values (find-lexically-apparent-fun
+               thing "as the argument to FUNCTION")
+              nil)))
+
+(def-ir1-translator %%allocate-closures ((&rest leaves) start next result)
+  (aver (eq result 'nil))
+  (let ((lambdas leaves))
+    (ir1-convert start next result `(%allocate-closures ',lambdas))
+    (let ((allocator (node-dest (ctran-next start))))
+      (dolist (lambda lambdas)
+        (setf (functional-allocator lambda) allocator)))))
+
+(defmacro with-fun-name-leaf ((leaf thing start) &body body)
+  `(multiple-value-bind (,leaf allocate-p) (fun-name-leaf ,thing)
+     (if allocate-p
+       (let ((.new-start. (make-ctran)))
+         (ir1-convert ,start .new-start. nil `(%%allocate-closures ,leaf))
+         (let ((,start .new-start.))
+           ,@body))
+       (locally
+           ,@body))))
 
 (def-ir1-translator function ((thing) start next result)
   #!+sb-doc
   "FUNCTION Name
   Return the lexically apparent definition of the function Name. Name may also
   be a lambda expression."
-  (reference-leaf start next result (fun-name-leaf thing)))
+  (with-fun-name-leaf (leaf thing start)
+    (reference-leaf start next result leaf)))
 
 ;;;; FUNCALL
 
@@ -474,8 +505,8 @@
 
 (def-ir1-translator %funcall ((function &rest args) start next result)
   (if (and (consp function) (eq (car function) 'function))
-      (ir1-convert start next result
-                   `(,(fun-name-leaf (second function)) ,@args))
+      (with-fun-name-leaf (leaf (second function) start)
+        (ir1-convert start next result `(,leaf ,@args)))
       (let ((ctran (make-ctran))
             (fun-lvar (make-lvar)))
         (ir1-convert start ctran fun-lvar `(the function ,function))
@@ -555,15 +586,16 @@
              (binding* ((ctran (make-ctran))
                         (fun-lvar (make-lvar))
                         ((next result)
-                         (processing-decls (decls vars nil next result)
-                                           (let ((fun (ir1-convert-lambda-body
-                                                       forms
-                                                       vars
-                                                       :debug-name (debug-namify "LET S"
-                                                                                 bindings))))
-                                             (reference-leaf start ctran fun-lvar fun))
-                                           (values next result))))
-                       (ir1-convert-combination-args fun-lvar ctran next result values)))))
+                         (processing-decls (decls vars nil next result
+						  post-binding-lexenv)
+                           (let ((fun (ir1-convert-lambda-body
+                                       forms
+                                       vars
+				       :post-binding-lexenv post-binding-lexenv
+                                       :debug-name (debug-name 'let bindings))))
+                             (reference-leaf start ctran fun-lvar fun))
+                           (values next result))))
+               (ir1-convert-combination-args fun-lvar ctran next result values)))))
         (t
          (compiler-error "Malformed LET bindings: ~S." bindings))))
 
@@ -577,15 +609,16 @@
       (multiple-value-bind (forms decls)
           (parse-body body :doc-string-allowed nil)
         (multiple-value-bind (vars values) (extract-let-vars bindings 'let*)
-          (processing-decls (decls vars nil start next)
-                            (ir1-convert-aux-bindings start 
-                                                      next 
-                                                      result
-                                                      forms
-                                                      vars 
-                                                      values))))
+          (processing-decls (decls vars nil start next post-binding-lexenv)
+            (ir1-convert-aux-bindings start
+                                      next
+                                      result
+                                      forms
+                                      vars
+                                      values
+				      post-binding-lexenv))))
       (compiler-error "Malformed LET* bindings: ~S." bindings)))
-  
+
 ;;; logic shared between IR1 translators for LOCALLY, MACROLET,
 ;;; and SYMBOL-MACROLET
 ;;;
@@ -637,6 +670,29 @@
 		     . ,forms))))))
     (values (names) (defs))))
 
+(defun ir1-convert-fbindings (start next result funs body)
+  (let ((ctran (make-ctran))
+        (dx-p (find-if #'leaf-dynamic-extent funs)))
+    (when dx-p
+      (ctran-starts-block ctran)
+      (ctran-starts-block next))
+    (ir1-convert start ctran nil `(%%allocate-closures ,@funs))
+    (cond (dx-p
+           (let* ((dummy (make-ctran))
+                  (entry (make-entry))
+                  (cleanup (make-cleanup :kind :dynamic-extent
+                                         :mess-up entry
+                                         :info (list (node-dest
+                                                      (ctran-next start))))))
+             (push entry (lambda-entries (lexenv-lambda *lexenv*)))
+             (setf (entry-cleanup entry) cleanup)
+             (link-node-to-previous-ctran entry ctran)
+             (use-ctran entry dummy)
+
+             (let ((*lexenv* (make-lexenv :cleanup cleanup)))
+               (ir1-convert-progn-body dummy next result body))))
+          (t (ir1-convert-progn-body ctran next result body)))))
+
 (def-ir1-translator flet ((definitions &body body)
 			  start next result)
   #!+sb-doc
@@ -651,15 +707,11 @@
       (let ((fvars (mapcar (lambda (n d)
                              (ir1-convert-lambda d
                                                  :source-name n
-                                                 :debug-name (debug-namify
-                                                              "FLET " n)))
+                                                 :debug-name (debug-name 'flet n)))
                            names defs)))
         (processing-decls (decls nil fvars next result)
           (let ((*lexenv* (make-lexenv :funs (pairlis names fvars))))
-            (ir1-convert-progn-body start 
-                                    next 
-                                    result
-                                    forms)))))))
+            (ir1-convert-fbindings start next result fvars forms)))))))
 
 (def-ir1-translator labels ((definitions &body body) start next result)
   #!+sb-doc
@@ -675,8 +727,8 @@
              (placeholder-funs (mapcar (lambda (name)
                                          (make-functional
                                           :%source-name name
-                                          :%debug-name (debug-namify
-                                                        "LABELS placeholder "
+                                          :%debug-name (debug-name 
+                                                        'labels-placeholder 
                                                         name)))
                                        names))
              ;; (like PAIRLIS but guaranteed to preserve ordering:)
@@ -688,17 +740,16 @@
                 (mapcar (lambda (name def)
                           (ir1-convert-lambda def
                                               :source-name name
-                                              :debug-name (debug-namify
-                                                           "LABELS " name)))
+                                              :debug-name (debug-name 'labels name)))
                         names defs))))
-        
+
         ;; Modify all the references to the dummy function leaves so
         ;; that they point to the real function leaves.
         (loop for real-fun in real-funs and
               placeholder-cons in placeholder-fenv do
               (substitute-leaf real-fun (cdr placeholder-cons))
               (setf (cdr placeholder-cons) real-fun))
-        
+
         ;; Voila.
         (processing-decls (decls nil real-funs next result)
           (let ((*lexenv* (make-lexenv
@@ -707,10 +758,7 @@
                            ;; lexical environment is used for inline
                            ;; expansion we'll get the right functions.
                            :funs (pairlis names real-funs))))
-            (ir1-convert-progn-body start 
-                                    next 
-                                    result
-                                    forms)))))))
+            (ir1-convert-fbindings start next result real-funs forms)))))))
 
 
 ;;;; the THE special operator, and friends
@@ -788,7 +836,7 @@
 		  name)))
 	     (setq-var start next result leaf (second things)))
 	    (cons
-	     (aver (eq (car leaf) 'MACRO))
+	     (aver (eq (car leaf) 'macro))
              ;; FIXME: [Free] type declaration. -- APD, 2002-01-26
 	     (ir1-convert start next result
                           `(setf ,(cdr leaf) ,(second things))))
@@ -866,9 +914,11 @@
                (ir1-convert-lambda
                 `(lambda ()
                    (return-from ,tag (%unknown-values)))
-                :debug-name (debug-namify "escape function for " tag)))))
+                :debug-name (debug-name 'escape-fun tag))))
+        (ctran (make-ctran)))
     (setf (functional-kind fun) :escape)
-    (reference-leaf start next result fun)))
+    (ir1-convert start ctran nil `(%%allocate-closures ,fun))
+    (reference-leaf ctran next result fun)))
 
 ;;; Yet another special special form. This one looks up a local
 ;;; function and smashes it to a :CLEANUP function, as well as
