@@ -51,7 +51,7 @@
 		       (t
 			(storew null-tn ptr
 				cons-cdr-slot list-pointer-lowtag)))
-		 (assert (null (tn-ref-across things)))
+		 (aver (null (tn-ref-across things)))
 		 (move result res))))))))
 
 (define-vop (list list-or-list*)
@@ -108,7 +108,8 @@
 
 (define-vop (make-closure)
   (:args (function :to :save :scs (descriptor-reg)))
-  (:info length)
+  (:info length stack-allocate-p)
+  (:ignore stack-allocate-p)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
@@ -128,9 +129,8 @@
   (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
   (:generator 10
-    (with-fixed-allocation
-	(result pa-flag temp value-cell-header-widetag value-cell-size))
-    (storew value result value-cell-value-slot other-pointer-lowtag)))
+    (with-fixed-allocation (result pa-flag temp value-cell-header-widetag value-cell-size)
+      (storew value result value-cell-value-slot other-pointer-lowtag))))
 
 
 ;;;; Automatic allocators for primitive objects.
