@@ -319,7 +319,7 @@ default-value-8
 	    (move csp-tn ocfp-tn)
 	    
 	    (let ((defaults (defaults)))
-	      (assert defaults)
+	      (aver defaults)
 	      (assemble (*elsewhere*)
 		(emit-label default-stack-vals)
 		(do ((remaining defaults (cdr remaining)))
@@ -618,7 +618,7 @@ default-value-8
 ;;; the current frame.
 ;;;
 (defmacro define-full-call (name named return variable)
-  (assert (not (and variable (eq return :tail))))
+  (aver (not (and variable (eq return :tail))))
   `(define-vop (,name
 		,@(when (eq return :unknown)
 		    '(unknown-values-receiver)))
@@ -1041,7 +1041,7 @@ default-value-8
   (:temporary (:sc any-reg :offset nl0-offset) result)
   (:temporary (:sc any-reg :offset nl1-offset) count)
   (:temporary (:sc any-reg :offset nl2-offset) src)
-  (:temporary (:sc any-reg :offset nl4-offset) dst)
+  (:temporary (:sc any-reg :offset nl3-offset) dst)
   (:temporary (:sc descriptor-reg :offset l0-offset) temp)
   (:info fixed)
   (:generator 20
