@@ -102,11 +102,11 @@
 	       (make-result-state))))))
 
 (deftransform %alien-funcall ((function type &rest args))
-  (assert (sb!c::constant-lvar-p type))
+  (aver (sb!c::constant-lvar-p type))
   (let* ((type (sb!c::lvar-value type))
 	 (arg-types (alien-fun-type-arg-types type))
 	 (result-type (alien-fun-type-result-type type)))
-    (assert (= (length arg-types) (length args)))
+    (aver (= (length arg-types) (length args)))
     ;; We need to do something special for the following argument
     ;; types: single-float, double-float, and 64-bit integers.  For
     ;; results, we need something special for 64-bit integer results.
@@ -192,8 +192,7 @@
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 2
-    (inst li res (make-fixup (extern-alien-name foreign-symbol) 
-                             :foreign))))
+    (inst li res (make-fixup foreign-symbol :foreign))))
 
 #!+linkage-table
 (define-vop (foreign-symbol-dataref-address)
@@ -206,8 +205,7 @@
   (:result-types system-area-pointer)
   (:temporary (:scs (non-descriptor-reg)) addr)
   (:generator 2
-    (inst li addr (make-fixup (extern-alien-name foreign-symbol) 
-                              :foreign-dataref))
+    (inst li addr (make-fixup foreign-symbol :foreign-dataref))
     (loadw res addr)))
 
 (define-vop (call-out)
@@ -227,7 +225,7 @@
       (when cur-nfp
 	(store-stack-tn nfp-save cur-nfp))
       (move cfunc function)
-      (inst li temp (make-fixup (extern-alien-name "call_into_c") :foreign))
+      (inst li temp (make-fixup "call_into_c" :foreign))
       (inst jal lip temp)
       (inst nop)
       (when cur-nfp
