@@ -11,31 +11,6 @@
 
 (in-package "SB!VM")
 
-;;; We can load/store into fp registers through the top of stack
-;;; %st(0) (fr0 here). Loads imply a push to an empty register which
-;;; then changes all the reg numbers. These macros help manage that.
-
-;;; Use this when we don't have to load anything. It preserves old tos
-;;; value, but probably destroys tn with operation.
-(defmacro with-tn@fp-top((tn) &body body)
-  `(progn
-    (unless (zerop (tn-offset ,tn))
-      (inst fxch ,tn))
-    ,@body
-    (unless (zerop (tn-offset ,tn))
-      (inst fxch ,tn))))
-
-;;; Use this to prepare for load of new value from memory. This
-;;; changes the register numbering so the next instruction had better
-;;; be a FP load from memory; a register load from another register
-;;; will probably be loading the wrong register!
-(defmacro with-empty-tn@fp-top((tn) &body body)
-  `(progn
-    (inst fstp ,tn)
-    ,@body
-    (unless (zerop (tn-offset ,tn))
-      (inst fxch ,tn))))		; save into new dest and restore st(0)
-
 ;;;; instruction-like macros
 
 (defmacro move (dst src)
@@ -48,6 +23,8 @@
 
 (defmacro make-ea-for-object-slot (ptr slot lowtag)
   `(make-ea :qword :base ,ptr :disp (- (* ,slot n-word-bytes) ,lowtag)))
+(defmacro make-ea-for-object-slot-half (ptr slot lowtag)
+  `(make-ea :dword :base ,ptr :disp (- (* ,slot n-word-bytes) ,lowtag)))
 
 (defmacro loadw (value ptr &optional (slot 0) (lowtag 0))
   `(inst mov ,value (make-ea-for-object-slot ,ptr ,slot ,lowtag)))
@@ -55,12 +32,12 @@
 (defmacro storew (value ptr &optional (slot 0) (lowtag 0))
   (once-only ((value value))
     `(cond ((and (integerp ,value) 
-		 (not (typep ,value 
-			     '(or (signed-byte 32) (unsigned-byte 32)))))
+		 (not (typep ,value '(signed-byte 32))))
 	    (multiple-value-bind (lo hi) (dwords-for-quad ,value)
-	      (inst mov (make-ea-for-object-slot ,ptr ,slot ,lowtag) lo)
-	      (inst mov (make-ea-for-object-slot ,ptr (floor (+ ,slot 0.5))
-						 ,lowtag)   hi)))
+	      (inst mov (make-ea-for-object-slot-half
+			 ,ptr ,slot ,lowtag) lo)
+	      (inst mov (make-ea-for-object-slot-half
+			 ,ptr (+ ,slot 1/2) ,lowtag) hi)))
 	   (t
 	    (inst mov (make-ea-for-object-slot ,ptr ,slot ,lowtag) ,value)))))
 
@@ -147,13 +124,75 @@
 ;;; node-var then it is used to make an appropriate speed vs size
 ;;; decision.
 
+(defun allocation-dynamic-extent (alloc-tn size)
+  (inst sub rsp-tn size)
+  ;; see comment in x86/macros.lisp implementation of this
+  (inst and rsp-tn #.(lognot lowtag-mask))
+  (aver (not (location= alloc-tn rsp-tn)))
+  (inst mov alloc-tn rsp-tn)
+  (values))
+
 ;;; This macro should only be used inside a pseudo-atomic section,
 ;;; which should also cover subsequent initialization of the
 ;;; object.
+(defun allocation-tramp (alloc-tn size &optional ignored)
+  (declare (ignore ignored))
+  (inst push size)
+  (inst lea r13-tn (make-ea :qword
+			    :disp (make-fixup "alloc_tramp" :foreign)))
+  (inst call r13-tn)
+  (inst pop alloc-tn)
+  (values))
+
+(defun allocation (alloc-tn size &optional ignored dynamic-extent)
+  (declare (ignore ignored))
+  (when dynamic-extent
+    (allocation-dynamic-extent alloc-tn size)
+    (return-from allocation (values)))
+  (let ((NOT-INLINE (gen-label))
+	(DONE (gen-label))
+	;; Yuck.
+	(in-elsewhere (eq *elsewhere* sb!assem::**current-segment**))
+	(free-pointer
+	 (make-ea :qword :disp 
+		  #!+sb-thread (* n-word-bytes thread-alloc-region-slot)
+		  #!-sb-thread (make-fixup "boxed_region" :foreign)
+		  :scale 1))		; thread->alloc_region.free_pointer
+	(end-addr 
+	 (make-ea :qword :disp
+		  #!+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
+		  #!-sb-thread (make-fixup "boxed_region" :foreign 8)
+		  :scale 1)))		; thread->alloc_region.end_addr
+    (cond (in-elsewhere
+	   (allocation-tramp alloc-tn size))
+	  (t
+	   (unless (and (tn-p size) (location= alloc-tn size))
+	     (inst mov alloc-tn size))
+	   #!+sb-thread (inst fs-segment-prefix)
+	   (inst add alloc-tn free-pointer)
+	   #!+sb-thread (inst fs-segment-prefix)
+	   (inst cmp end-addr alloc-tn)
+	   (inst jmp :be NOT-INLINE)
+	   #!+sb-thread (inst fs-segment-prefix)
+	   (inst xchg free-pointer alloc-tn)
+	   (emit-label DONE)
+	   (assemble (*elsewhere*)
+	     (emit-label NOT-INLINE)
+	     (cond ((numberp size)
+		    (allocation-tramp alloc-tn size))
+		   (t
+		    (inst sub alloc-tn free-pointer)
+		    (allocation-tramp alloc-tn alloc-tn)))
+	     (inst jmp DONE))
+	   (values)))))
+
+#+nil
 (defun allocation (alloc-tn size &optional ignored)
   (declare (ignore ignored))
   (inst push size)
-  (inst call (make-fixup (extern-alien-name "alloc_tramp") :foreign))
+  (inst lea r13-tn (make-ea :qword
+			    :disp (make-fixup "alloc_tramp" :foreign)))
+  (inst call r13-tn)
   (inst pop alloc-tn)
   (values))
 
@@ -161,14 +200,17 @@
 ;;; header having the specified WIDETAG value. The result is placed in
 ;;; RESULT-TN.
 (defmacro with-fixed-allocation ((result-tn widetag size &optional inline)
-				 &rest forms)
-  `(pseudo-atomic
-    (allocation ,result-tn (pad-data-block ,size) ,inline)
-    (storew (logior (ash (1- ,size) n-widetag-bits) ,widetag)
-	    ,result-tn)
-    (inst lea ,result-tn
-     (make-ea :byte :base ,result-tn :disp other-pointer-lowtag))
-    ,@forms))
+				 &body forms)
+  (unless forms
+    (bug "empty &body in WITH-FIXED-ALLOCATION"))
+  (once-only ((result-tn result-tn) (size size))
+    `(pseudo-atomic
+      (allocation ,result-tn (pad-data-block ,size) ,inline)
+      (storew (logior (ash (1- ,size) n-widetag-bits) ,widetag)
+	      ,result-tn)
+      (inst lea ,result-tn
+	    (make-ea :qword :base ,result-tn :disp other-pointer-lowtag))
+      ,@forms)))
 
 ;;;; error code
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
@@ -227,6 +269,12 @@
 ;;; the ATOMIC bit is bit 0, the INTERRUPTED bit is bit 1, and you check
 ;;; the C flag after the shift to see whether you were interrupted.
 
+;;; FIXME: THIS NAME IS BACKWARDS!
+(defmacro maybe-pseudo-atomic (really-p &body body)
+  `(if ,really-p
+       (progn ,@body)
+       (pseudo-atomic ,@body)))
+
 (defmacro pseudo-atomic (&rest forms)
   (with-unique-names (label)
     `(let ((,label (gen-label)))
