@@ -269,7 +269,9 @@
     sub-gc
     sb!kernel::internal-error
     sb!kernel::control-stack-exhausted-error
-    sb!kernel::undefined-alien-error
+    sb!kernel::undefined-alien-variable-error
+    sb!kernel::undefined-alien-function-error
+    sb!kernel::memory-fault-error
     sb!di::handle-breakpoint
     fdefinition-object
     #!+sb-thread sb!thread::handle-thread-exit
@@ -304,6 +306,8 @@
     *control-stack-start*
     *control-stack-end*
 
+    *need-to-collect-garbage*
+
     ;; the floating point constants
     *fp-constant-0d0*
     *fp-constant-1d0*
