@@ -59,9 +59,9 @@
 
 (defvar sb-pcl::*internal-pcl-generalized-fun-name-symbols* nil)
 
-(defmacro define-internal-pcl-function-name-syntax (name &rest rest)
+(defmacro define-internal-pcl-function-name-syntax (name &body body)
   `(progn
-     (define-function-name-syntax ,name ,@rest)
+     (define-function-name-syntax ,name ,@body)
      (pushnew ',name sb-pcl::*internal-pcl-generalized-fun-name-symbols*)))
 
 (define-internal-pcl-function-name-syntax sb-pcl::class-predicate (list)
@@ -79,6 +79,15 @@
 		 (symbolp class))
 	(values t slot)))))
 
+(define-internal-pcl-function-name-syntax sb-pcl::fast-method (list)
+  (valid-function-name-p (cadr list)))
+
+(define-internal-pcl-function-name-syntax sb-pcl::slow-method (list)
+  (valid-function-name-p (cadr list)))
+
+(define-internal-pcl-function-name-syntax sb-pcl::ctor (list)
+  (valid-function-name-p (cadr list)))
+
 (defun sb-pcl::random-documentation (name type)
   (cdr (assoc type (info :random-documentation :stuff name))))
 
