@@ -306,7 +306,8 @@
 				 ;; CMU CL code did. -- WHN 20000715
 				 '(t instance
 				     funcallable-instance
-				     function stream)))
+				     function stream 
+				     file-stream string-stream)))
 		       sb-kernel::*built-in-classes*))))
 (/noshow "done setting up SB-PCL::*BUILT-IN-CLASSES*")
 
@@ -327,6 +328,12 @@
 (defclass stream (instance) ()
   (:metaclass built-in-class))
 
+(defclass file-stream (stream) ()
+  (:metaclass built-in-class))
+
+(defclass string-stream (stream) ()
+  (:metaclass built-in-class))
+
 (defclass slot-object (t) ()
   (:metaclass slot-class))
 
@@ -693,7 +700,11 @@
     :initarg :method-combination
     :accessor generic-function-method-combination)
    (declarations
+    ;; KLUDGE: AMOP specifies :DECLARATIONS, while ANSI specifies
+    ;; :DECLARE.  Allow either (but FIXME: maybe a note or a warning
+    ;; might be appropriate).
     :initarg :declarations
+    :initarg :declare
     :initform ()
     :accessor generic-function-declarations)
    (arg-info
