@@ -174,7 +174,8 @@
 	     (setf (find-classoid name) nil))
 	   (when (or (eq *boot-state* 'complete)
 		     (eq *boot-state* 'braid))
-	     (when (and new-value (class-wrapper new-value))
+	     (when (and new-value (class-wrapper new-value)
+                        (class-predicate-name new-value))
 	       (setf (find-class-cell-predicate cell)
 		     (fdefinition (class-predicate-name new-value))))
 	     (update-ctors 'setf-find-class :class new-value :name name))
