@@ -653,9 +653,10 @@
 
 
 (defun error-need-at-least-n-args (function n)
-  (error "~@<The function ~2I~_~S ~I~_requires at least ~W argument~:P.~:>"
-	 function
-	 n))
+  (error 'simple-program-error
+	 :format-control "~@<The function ~2I~_~S ~I~_requires ~
+                          at least ~W argument~:P.~:>"
+	 :format-arguments (list function n)))
 
 (defun types-from-args (generic-function arguments &optional type-modifier)
   (multiple-value-bind (nreq applyp metatypes nkeys arg-info)
