@@ -1044,7 +1044,7 @@
 (defun name-method-lambda (method-lambda)
   (let ((method-name (body-method-name (cddr method-lambda))))
     (if method-name
-	`(named-lambda ,method-name ,(rest method-lambda))
+	`(named-lambda (slow-method ,method-name) ,(rest method-lambda))
 	method-lambda)))
 
 (defun make-method-initargs-form-internal (method-lambda initargs env)
@@ -1093,7 +1093,8 @@
 	:fast-function
 	(,(if (body-method-name body) 'named-lambda 'lambda)
 	  ,@(when (body-method-name body)
-		  (list (body-method-name body))) ; function name
+                  ;; function name
+		  (list (cons 'fast-method (body-method-name body))))
 	  (.pv-cell. .next-method-call. ,@args+rest-arg) ; function args
 	  ;; body of the function
 	  (declare (ignorable .pv-cell. .next-method-call.))
@@ -1150,13 +1151,7 @@
 		    (apply fmf pv-cell nmc (nconc args (list rest))))
 		  (apply fmf pv-cell nmc method-args)))))
     (let* ((fname (method-function-get fmf :name))
-	   (name `(,(or (get (car fname) 'method-sym)
-			(setf (get (car fname) 'method-sym)
-			      (let ((str (symbol-name (car fname))))
-				(if (string= "FAST-" str :end2 5)
-				    (format-symbol *pcl-package* (subseq str 5))
-				    (car fname)))))
-		    ,@(cdr fname))))
+	   (name (cons 'slow-method (cdr fname))))
       (set-fun-name method-function name))
     (setf (method-function-get method-function :fast-function) fmf)
     method-function))
