@@ -26,7 +26,7 @@
 #include "genesis/fdefn.h"
 
 #define REAL_LRA_SLOT 0
-#ifndef LISP_FEATURE_X86
+#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 #define KNOWN_RETURN_P_SLOT 1
 #define BOGUS_LRA_CONSTANTS 2
 #else
@@ -65,13 +65,13 @@ void breakpoint_do_displaced_inst(os_context_t* context,
      *
      * -dan 2001.08.09 */
 
-#if (defined(sparc) && defined (solaris))
+#if (defined(LISP_FEATURE_SPARC) && defined (solaris))
     undo_fake_foreign_function_call(context);
 #endif
     arch_do_displaced_inst(context, orig_inst);
 }
 
-#ifndef LISP_FEATURE_X86
+#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 static lispobj find_code(os_context_t *context)
 {
 #ifdef reg_CODE
@@ -91,9 +91,7 @@ static lispobj find_code(os_context_t *context)
     return NIL;
 #endif
 }
-#endif
-
-#ifdef LISP_FEATURE_X86
+#else
 static lispobj find_code(os_context_t *context)
 {
     lispobj codeptr =
@@ -107,7 +105,7 @@ static lispobj find_code(os_context_t *context)
 }
 #endif
 
-static int compute_offset(os_context_t *context, lispobj code)
+static long compute_offset(os_context_t *context, lispobj code)
 {
     if (code == NIL)
 	return 0;
@@ -125,7 +123,7 @@ static int compute_offset(os_context_t *context, lispobj code)
 	if (pc < code_start)
 	    return 0;
 	else {
-	    int offset = pc - code_start;
+	    long offset = pc - code_start;
 	    if (offset >= codeptr->code_size)
 		return 0;
 	    else
@@ -133,28 +131,7 @@ static int compute_offset(os_context_t *context, lispobj code)
 	}
     }
 }
-/* FIXME: I can see no really good reason these couldn't be merged, but haven't
- * tried.  The sigprocmask() call would work just as well on alpha as it
- * presumably does on x86   -dan 2001.08.10
- */
-#ifndef LISP_FEATURE_X86
-void handle_breakpoint(int signal, siginfo_t *info, os_context_t *context)
-{
-    lispobj code;
 
-    fake_foreign_function_call(context);
-
-    code = find_code(context);
-    /* FIXME we're calling into Lisp with signals masked here.  Is this
-     * the right thing to do? */
-    funcall3(SymbolFunction(HANDLE_BREAKPOINT),
-	     compute_offset(context, code),
-	     code,
-	     alloc_sap(context));
-
-    undo_fake_foreign_function_call(context);
-}
-#else
 void handle_breakpoint(int signal, siginfo_t* info, os_context_t *context)
 {
     lispobj code, context_sap = alloc_sap(context);
@@ -165,7 +142,7 @@ void handle_breakpoint(int signal, siginfo_t* info, os_context_t *context)
 
     /* Don't disallow recursive breakpoint traps. Otherwise, we can't
      * use debugger breakpoints anywhere in here. */
-    sigprocmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+    thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
 
     funcall3(SymbolFunction(HANDLE_BREAKPOINT),
 	     compute_offset(context, code),
@@ -174,9 +151,8 @@ void handle_breakpoint(int signal, siginfo_t* info, os_context_t *context)
 
     undo_fake_foreign_function_call(context);
 }
-#endif
 
-#ifndef LISP_FEATURE_X86
+#if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
 void *handle_fun_end_breakpoint(int signal, siginfo_t *info,
 				os_context_t *context)
 {
@@ -188,8 +164,10 @@ void *handle_fun_end_breakpoint(int signal, siginfo_t *info,
     code = find_code(context);
     codeptr = (struct code *)native_pointer(code);
 
-    /* FIXME again, calling into Lisp with signals masked.  Is this
-     * sensible? */
+    /* Don't disallow recursive breakpoint traps. Otherwise, we can't
+     * use debugger breakpoints anywhere in here. */
+    thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+
     funcall3(SymbolFunction(HANDLE_BREAKPOINT),
 	     compute_offset(context, code),
 	     code,
@@ -218,7 +196,7 @@ void *handle_fun_end_breakpoint(int signal, siginfo_t *info,
 
     /* Don't disallow recursive breakpoint traps. Otherwise, we can't
      * use debugger breakpoints anywhere in here. */
-    sigprocmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+    thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
 
     funcall3(SymbolFunction(HANDLE_BREAKPOINT),
 	     compute_offset(context, code),
