@@ -90,19 +90,19 @@ zero_stack(void)
 
 
 void *
-gc_general_alloc(int bytes, int unboxed_p, int quick_p) {
+gc_general_alloc(long bytes, int unboxed_p, int quick_p) {
     lispobj *new=new_space_free_pointer;
     new_space_free_pointer+=(bytes/N_WORD_BYTES);
     return new;
 }
 
-lispobj  copy_large_unboxed_object(lispobj object, int nwords) {
+lispobj  copy_large_unboxed_object(lispobj object, long nwords) {
     return copy_object(object,nwords);
 }
-lispobj  copy_unboxed_object(lispobj object, int nwords) {
+lispobj  copy_unboxed_object(lispobj object, long nwords) {
     return copy_object(object,nwords);
 }
-lispobj  copy_large_object(lispobj object, int nwords) {
+lispobj  copy_large_object(lispobj object, long nwords) {
     return copy_object(object,nwords);
 }
 
@@ -140,7 +140,7 @@ collect_garbage(unsigned ignore)
      * from a signal handler (e.g. with the sigsegv gc_trigger stuff) */
     sigemptyset(&tmp);
     sigaddset_blockable(&tmp);
-    sigprocmask(SIG_BLOCK, &tmp, &old);
+    thread_sigmask(SIG_BLOCK, &tmp, &old);
 
     current_static_space_free_pointer =
 	(lispobj *) ((unsigned long)
@@ -237,7 +237,10 @@ collect_garbage(unsigned ignore)
     printf("Flipping spaces ...\n");
 #endif
 
-    os_zero((os_vm_address_t) current_dynamic_space,
+    /* Maybe FIXME: it's possible that we could significantly reduce 
+     * RSS by zeroing the from_space or madvise(MADV_DONTNEED) or 
+     * similar os-dependent tricks here */
+    os_zero((os_vm_address_t) from_space,
 	    (os_vm_size_t) DYNAMIC_SPACE_SIZE);
 
     current_dynamic_space = new_space;
@@ -248,13 +251,15 @@ collect_garbage(unsigned ignore)
 #endif
     size_retained = (new_space_free_pointer - new_space) * sizeof(lispobj);
 
+    os_flush_icache((os_vm_address_t)new_space, size_retained);
+
     /* Zero stack. */
 #ifdef PRINTNOISE
     printf("Zeroing empty part of control stack ...\n");
 #endif
     zero_stack();
     set_auto_gc_trigger(size_retained+bytes_consed_between_gcs);
-    sigprocmask(SIG_SETMASK, &old, 0);
+    thread_sigmask(SIG_SETMASK, &old, 0);
 
 
 #ifdef PRINTNOISE
@@ -280,10 +285,6 @@ collect_garbage(unsigned ignore)
 	
     printf("%10.2f M bytes/sec collected.\n", gc_rate);
 #endif
-    /* os_flush_icache((os_vm_address_t) 0, sizeof(unsigned long)); */
-    /* Maybe FIXME: it's possible that we could significantly reduce 
-     * RSS by zeroing the from_space or madvise(MADV_DONTNEED) or 
-     * similar os-dependent tricks here */
 }
 
 
@@ -419,8 +420,6 @@ void scavenge_interrupt_contexts(void)
     os_context_t *context;
 
     struct thread *th=arch_os_get_current_thread();
-    struct interrupt_data *data=
-	th ? th->interrupt_data : global_interrupt_data;
 
     index = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,0));
 
@@ -494,48 +493,10 @@ print_garbage(lispobj *from_space, lispobj *from_space_free_pointer)
     printf("%d total words not copied.\n", total_words_not_copied);
 }
 
-
-/* code and code-related objects */
-
-/* FIXME (1) this could probably be defined using something like
- *  sizeof(lispobj)*floor(sizeof(struct simple_fun)/sizeof(lispobj))
- *    -  FUN_POINTER_LOWTAG
- * as I'm reasonably sure that simple_fun->code must always be the 
- * last slot in the object 
-
- * FIXME (2) it also appears in purify.c, and it has a different value
- * for SPARC users in that bit
- */
-
-#define FUN_RAW_ADDR_OFFSET (6*sizeof(lispobj) - FUN_POINTER_LOWTAG)
-
-/* Note: on the sparc we don't have to do anything special for fdefns, */
-/* 'cause the raw-addr has a function lowtag. */
-#ifndef LISP_FEATURE_SPARC
-static int
-scav_fdefn(lispobj *where, lispobj object)
-{
-    struct fdefn *fdefn;
-
-    fdefn = (struct fdefn *)where;
-    
-    if ((char *)(fdefn->fun + FUN_RAW_ADDR_OFFSET) 
-	== (char *)((unsigned long)(fdefn->raw_addr))) {
-        scavenge(where + 1, sizeof(struct fdefn)/sizeof(lispobj) - 1);
-        fdefn->raw_addr =
-            (u32) ((char *) LOW_WORD(fdefn->fun)) + FUN_RAW_ADDR_OFFSET;
-        return sizeof(struct fdefn) / sizeof(lispobj);
-    }
-    else
-        return 1;
-}
-#endif
-
-
 
 /* vector-like objects */
 
-static int
+static long
 scav_vector(lispobj *where, lispobj object)
 {
     if (HeaderValue(object) == subtype_VectorValidHashing) {
@@ -552,7 +513,7 @@ scav_vector(lispobj *where, lispobj object)
 #define WEAK_POINTER_NWORDS \
 	CEILING((sizeof(struct weak_pointer) / sizeof(lispobj)), 2)
 
-static int
+static long
 scav_weak_pointer(lispobj *where, lispobj object)
 {
     /* Do not let GC scavenge the value slot of the weak pointer */
@@ -569,9 +530,9 @@ search_read_only_space(void *pointer)
     lispobj* end = (lispobj*)SymbolValue(READ_ONLY_SPACE_FREE_POINTER,0);
     if ((pointer < (void *)start) || (pointer >= (void *)end))
 	return NULL;
-    return (search_space(start, 
-			 (((lispobj *)pointer)+2)-start, 
-			 (lispobj *)pointer));
+    return (gc_search_space(start, 
+			    (((lispobj *)pointer)+2)-start, 
+			    (lispobj *)pointer));
 }
 
 lispobj *
@@ -581,9 +542,9 @@ search_static_space(void *pointer)
     lispobj* end = (lispobj*)SymbolValue(STATIC_SPACE_FREE_POINTER,0);
     if ((pointer < (void *)start) || (pointer >= (void *)end))
 	return NULL;
-    return (search_space(start, 
-			 (((lispobj *)pointer)+2)-start, 
-			 (lispobj *)pointer));
+    return (gc_search_space(start, 
+			    (((lispobj *)pointer)+2)-start, 
+			    (lispobj *)pointer));
 }
 
 lispobj *
@@ -593,9 +554,9 @@ search_dynamic_space(void *pointer)
     lispobj *end = (lispobj *) dynamic_space_free_pointer;
     if ((pointer < (void *)start) || (pointer >= (void *)end))
 	return NULL;
-    return (search_space(start, 
-			 (((lispobj *)pointer)+2)-start, 
-			 (lispobj *)pointer));
+    return (gc_search_space(start, 
+			    (((lispobj *)pointer)+2)-start, 
+			    (lispobj *)pointer));
 }
 
 /* initialization.  if gc_init can be moved to after core load, we could
@@ -637,16 +598,16 @@ void set_auto_gc_trigger(os_vm_size_t dynamic_usage)
 
     if (addr < (os_vm_address_t)dynamic_space_free_pointer) {
 	fprintf(stderr,
-	   "set_auto_gc_trigger: tried to set gc trigger too low! (%d < %p)\n",
-		(unsigned int)dynamic_usage,
-		(os_vm_address_t)dynamic_space_free_pointer
-		- (os_vm_address_t)current_dynamic_space);
+	   "set_auto_gc_trigger: tried to set gc trigger too low! (%ld < 0x%08lx)\n",
+		(unsigned long)dynamic_usage,
+		(unsigned long)((os_vm_address_t)dynamic_space_free_pointer
+				- (os_vm_address_t)current_dynamic_space));
 	lose("lost");
     }
     else if (length < 0) {
 	fprintf(stderr,
-		"set_auto_gc_trigger: tried to set gc trigger too high! (%p)\n",
-		dynamic_usage);
+		"set_auto_gc_trigger: tried to set gc trigger too high! (0x%08lx)\n",
+		(unsigned long)dynamic_usage);
 	lose("lost");
     }
 
