@@ -9,14 +9,14 @@ struct alloc_region {
     void  *end_addr; /* pointer to the byte after the last usable byte */
 
     /* These are needed when closing the region. */
-    int  first_page;
-    int  last_page;
+    long  first_page;
+    long  last_page;
     void  *start_addr;
 };
 
 extern struct alloc_region  boxed_region;
 extern struct alloc_region  unboxed_region;
-extern int from_space, new_space;
+extern long from_space, new_space;
 extern struct weak_pointer *weak_pointers;
 
 extern void *current_region_free_pointer;
