@@ -28,10 +28,12 @@
 #include "os.h"
 #include "arch.h"
 #include "globals.h"
-#include "sbcl.h"
 #include "interrupt.h"
 #include "interr.h"
 #include "lispregs.h"
+#include "runtime.h"
+#include "genesis/static-symbols.h"
+#include "genesis/fdefn.h"
 #include <sys/socket.h>
 #include <sys/utsname.h>
 
@@ -47,7 +49,7 @@
 #include "thread.h"
 size_t os_vm_page_size;
 
-#ifdef LISP_FEATURE_SB_FUTEX
+#ifdef LISP_FEATURE_SB_THREAD
 #include <linux/unistd.h>
 #include <errno.h>
 
@@ -71,13 +73,16 @@ _syscall4(int,sys_futex,
 #include "gc.h"
 
 int linux_sparc_siginfo_bug = 0;
-int linux_supports_futex=0;
+int linux_no_threads_p = 0;
 
-void os_init(void)
+void
+os_init(void)
 {
     /* Conduct various version checks: do we have enough mmap(), is
      * this a sparc running 2.2, can we do threads? */
+#ifdef LISP_FEATURE_SB_THREAD
     int *futex=0;
+#endif
     struct utsname name;
     int major_version;
     int minor_version;
@@ -92,17 +97,16 @@ void os_init(void)
 	     major_version);
     }
     if (!(major_version>2 || minor_version >= 4)) {
-#ifdef LISP_FEATURE_SB_THREAD
-	lose("linux kernel 2.4 required for thread-enabled SBCL");
-#endif
 #ifdef LISP_FEATURE_SPARC
 	FSHOW((stderr,"linux kernel %d.%d predates 2.4;\n enabling workarounds for SPARC kernel bugs in signal handling.\n", major_version,minor_version));
 	linux_sparc_siginfo_bug = 1;
 #endif
     }
-#ifdef LISP_FEATURE_SB_FUTEX
+#ifdef LISP_FEATURE_SB_THREAD
     futex_wait(futex,-1);
-    if(errno!=ENOSYS) linux_supports_futex=1;
+    if(errno==ENOSYS)  linux_no_threads_p = 1;
+    if(linux_no_threads_p) 
+	fprintf(stderr,"Linux with NPTL support (e.g. kernel 2.6 or newer) required for \nthread-enabled SBCL.  Disabling thread support.\n\n");
 #endif
     os_vm_page_size = getpagesize();
 }
@@ -157,17 +161,16 @@ os_invalidate(os_vm_address_t addr, os_vm_size_t len)
 os_vm_address_t
 os_map(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
 {
-    addr = mmap(addr, len,
-		OS_VM_PROT_ALL,
-		MAP_PRIVATE | MAP_FILE | MAP_FIXED,
-		fd, (off_t) offset);
+    os_vm_address_t actual;
 
-    if (addr == MAP_FAILED) {
+    actual = mmap(addr, len, OS_VM_PROT_ALL, MAP_PRIVATE | MAP_FIXED,
+		  fd, (off_t) offset);
+    if (actual == MAP_FAILED || (addr && (addr != actual))) {
 	perror("mmap");
 	lose("unexpected mmap(..) failure");
     }
 
-    return addr;
+    return actual;
 }
 
 void
@@ -178,30 +181,28 @@ os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
     }
 }
 
-/* FIXME: Now that FOO_END, rather than FOO_SIZE, is the fundamental
- * description of a space, we could probably punt this and just do
- * (FOO_START <= x && x < FOO_END) everywhere it's called. */
-static boolean
-in_range_p(os_vm_address_t a, lispobj sbeg, size_t slen)
-{
-    char* beg = (char*)((long)sbeg);
-    char* end = (char*)((long)sbeg) + slen;
-    char* adr = (char*)a;
-    return (adr >= beg && adr < end);
-}
-
 boolean
 is_valid_lisp_addr(os_vm_address_t addr)
 {
     struct thread *th;
-    if(in_range_p(addr, READ_ONLY_SPACE_START, READ_ONLY_SPACE_SIZE) ||
-       in_range_p(addr, STATIC_SPACE_START   , STATIC_SPACE_SIZE) ||
-       in_range_p(addr, DYNAMIC_SPACE_START  , DYNAMIC_SPACE_SIZE))
+    size_t ad = (size_t) addr;
+ 
+    if ((READ_ONLY_SPACE_START <= ad && ad < READ_ONLY_SPACE_END)
+	|| (STATIC_SPACE_START <= ad && ad < STATIC_SPACE_END)
+#if defined LISP_FEATURE_GENCGC
+	|| (DYNAMIC_SPACE_START <= ad && ad < DYNAMIC_SPACE_END)
+#else
+	|| (DYNAMIC_0_SPACE_START <= ad && ad < DYNAMIC_0_SPACE_END)
+	|| (DYNAMIC_1_SPACE_START <= ad && ad < DYNAMIC_1_SPACE_END)
+#endif
+	)
 	return 1;
     for_each_thread(th) {
-	if((th->control_stack_start <= addr) && (addr < th->control_stack_end))
+	if((size_t)(th->control_stack_start) <= ad
+	   && ad < (size_t)(th->control_stack_end))
 	    return 1;
-	if(in_range_p(addr, th->binding_stack_start, BINDING_STACK_SIZE))
+	if((size_t)(th->binding_stack_start) <= ad
+	   && ad < (size_t)(th->binding_stack_start + BINDING_STACK_SIZE))
 	    return 1;
     }
     return 0;
@@ -218,14 +219,18 @@ is_valid_lisp_addr(os_vm_address_t addr)
  * The GENCGC needs to be hooked into whatever signal is raised for
  * page fault on this OS.
  */
-void
+static void
 sigsegv_handler(int signal, siginfo_t *info, void* void_context)
 {
     os_context_t *context = arch_os_get_context(&void_context);
     void* fault_addr = (void*)info->si_addr;
     if (!gencgc_handle_wp_violation(fault_addr)) 
 	if(!handle_guard_page_triggered(context,fault_addr))
-	    interrupt_handle_now(signal, info, void_context);
+#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
+            arrange_return_to_lisp_function(context, SymbolFunction(MEMORY_FAULT_ERROR));
+#else
+            interrupt_handle_now(signal, info, context);
+#endif
 }
 
 #else
@@ -234,41 +239,29 @@ static void
 sigsegv_handler(int signal, siginfo_t *info, void* void_context)
 {
     os_context_t *context = arch_os_get_context(&void_context);
-    os_vm_address_t addr;
+    os_vm_address_t addr = arch_get_bad_addr(signal,info,context);
 
-    addr = arch_get_bad_addr(signal,info,context);
+#ifdef LISP_FEATURE_ALPHA
+    /* Alpha stuff: This is the end of a pseudo-atomic section during
+       which a signal was received.  We must deal with the pending
+       interrupt (see also interrupt.c, ../code/interrupt.lisp)
+
+       (how we got here: when interrupting, we set bit 63 in reg_ALLOC.
+       At the end of the atomic section we tried to write to reg_ALLOC,
+       got a SIGSEGV (there's nothing mapped there) so ended up here. */
     if (addr != NULL && 
 	*os_context_register_addr(context,reg_ALLOC) & (1L<<63)){
-	
-	/* Alpha stuff: This is the end of a pseudo-atomic section
-	 * during which a signal was received.  We must deal with the
-	 * pending interrupt (see also interrupt.c,
-	 * ../code/interrupt.lisp)
-	 */
-	/* (how we got here: when interrupting, we set bit 63 in
-	 * reg_Alloc.  At the end of the atomic section we tried to
-	 * write to reg_ALLOC, got a SIGSEGV (there's nothing mapped
-	 * there) so ended up here
-	 */
 	*os_context_register_addr(context,reg_ALLOC) -= (1L<<63);
 	interrupt_handle_pending(context);
-    } else {
-	if(!interrupt_maybe_gc(signal, info, context))
-	    if(!handle_guard_page_triggered(context,addr))
-		interrupt_handle_now(signal, info, context);
+	return;
     }
-}
 #endif
 
-void sigcont_handler(int signal, siginfo_t *info, void *void_context)
-{
-    /* We need to have a handler installed for this signal so that
-     * sigwaitinfo() for it actually returns at the appropriate time.
-     * We don't need it to actually do anything.  This mkes it
-     * possibly the only signal handler in SBCL that doesn't depend on
-     * not-guaranteed-by-POSIX features 
-     */    
+    if(!interrupt_maybe_gc(signal, info, context))
+	if(!handle_guard_page_triggered(context,addr))
+	    interrupt_handle_now(signal, info, context);
 }
+#endif
 
 void
 os_install_interrupt_handlers(void)
@@ -280,20 +273,20 @@ os_install_interrupt_handlers(void)
 						 interrupt_thread_handler);
     undoably_install_low_level_interrupt_handler(SIG_STOP_FOR_GC,
 						 sig_stop_for_gc_handler);
-    undoably_install_low_level_interrupt_handler(SIG_THREAD_EXIT,
-						 thread_exit_handler);
-    if(!linux_supports_futex)
-	undoably_install_low_level_interrupt_handler(SIG_DEQUEUE,
-						     sigcont_handler);
 #endif
 }
 
-#ifdef LISP_FEATURE_SB_FUTEX
-int futex_wait(int *lock_word, int oldval) {
+#ifdef LISP_FEATURE_SB_THREAD
+int
+futex_wait(int *lock_word, int oldval)
+{
     int t= sys_futex(lock_word,FUTEX_WAIT,oldval, 0);
     return t;
 }
-int futex_wake(int *lock_word, int n){
+
+int
+futex_wake(int *lock_word, int n)
+{
     return sys_futex(lock_word,FUTEX_WAKE,n,0);
 }
 #endif
