@@ -195,7 +195,7 @@ static void newline(char *label)
 
 static void brief_fixnum(lispobj obj)
 {
-#ifndef alpha
+#ifndef LISP_FEATURE_ALPHA
     printf("%ld", ((long)obj)>>2);
 #else
     printf("%d", ((s32)obj)>>2);
@@ -204,7 +204,7 @@ static void brief_fixnum(lispobj obj)
 
 static void print_fixnum(lispobj obj)
 {
-#ifndef alpha
+#ifndef LISP_FEATURE_ALPHA
     printf(": %ld", ((long)obj)>>2);
 #else
     printf(": %d", ((s32)obj)>>2);
@@ -439,7 +439,7 @@ static void print_otherptr(lispobj obj)
     if (!is_valid_lisp_addr((os_vm_address_t)obj)) {
 	printf("(invalid address)");
     } else {
-#ifndef alpha
+#ifndef LISP_FEATURE_ALPHA
         lispobj *ptr;
         unsigned long header;
         unsigned long length;
@@ -491,11 +491,12 @@ static void print_otherptr(lispobj obj)
                 print_slots(symbol_slots, count, ptr);
                 break;
 
+#if N_WORD_BITS == 32
             case SINGLE_FLOAT_WIDETAG:
                 NEWLINE_OR_RETURN;
                 printf("%g", ((struct single_float *)native_pointer(obj))->value);
                 break;
-
+#endif
             case DOUBLE_FLOAT_WIDETAG:
                 NEWLINE_OR_RETURN;
                 printf("%g", ((struct double_float *)native_pointer(obj))->value);
@@ -637,7 +638,7 @@ static void print_otherptr(lispobj obj)
 
             case SAP_WIDETAG:
                 NEWLINE_OR_RETURN;
-#ifndef alpha
+#ifndef LISP_FEATURE_ALPHA
                 printf("0x%08lx", (unsigned long) *ptr);
 #else
                 printf("0x%016lx", *(lispobj*)(ptr+1));
