@@ -97,7 +97,7 @@
 (define-alien-type enum.2 (enum nil (zero 0) (one 1) (two 2) (three 3)
                                     (four 4) (five 5) (six 6) (seven 7)
                                     (eight 8) (nine 9)))
-(with-alien ((integer-array (array integer 3)))
+(with-alien ((integer-array (array int 3)))
   (let ((enum-array (cast integer-array (array enum.2 3))))
     (setf (deref enum-array 0) 'three
           (deref enum-array 1) 'four)
@@ -107,7 +107,7 @@
 ;; The code that is used for mapping from integers to symbols depends on the
 ;; `density' of the set of used integers, so test with a sparse set as well.
 (define-alien-type enum.3 (enum nil (zero 0) (one 1) (k-one 1001) (k-two 1002)))
-(with-alien ((integer-array (array integer 3)))
+(with-alien ((integer-array (array int 3)))
   (let ((enum-array (cast integer-array (array enum.3 3))))
     (setf (deref enum-array 0) 'one
           (deref enum-array 1) 'k-one)
@@ -115,5 +115,15 @@
                                      (deref integer-array 1)))
     (assert (eql (deref enum-array 2) 'k-two))))
 
+;;; As reported by Baughn on #lisp, ALIEN-FUNCALL loops forever when
+;;; compiled with (DEBUG 3).
+(sb-kernel::values-specifier-type-cache-clear)
+(proclaim '(optimize (debug 3)))
+(let ((f (compile nil '(lambda (v)
+                        (sb-alien:alien-funcall (sb-alien:extern-alien "getenv"
+                                                 (function (c-string) c-string))
+                         v)))))
+  (assert (typep (funcall f "HOME") '(or string null))))
+
 ;;; success
 (quit :unix-status 104)
