@@ -1667,3 +1667,115 @@
        (type (simple-array (unsigned-byte 32) (*)) v))
       (setf (aref v 0) (* (* x #.(floor (ash 1 32) (* 11 80))) y))
       nil)))
+
+;;; Bug reported by Robert J. Macomber: instrumenting of more-entry
+;;; prevented open coding of %LISTIFY-REST-ARGS.
+(let ((f (compile nil '(lambda ()
+                        (declare (optimize (debug 3)))
+                        (with-simple-restart (blah "blah") (error "blah"))))))
+  (handler-bind ((error (lambda (c) (invoke-restart 'blah))))
+    (assert (equal (multiple-value-list (funcall f)) '(nil t)))))
+
+;;; Bug reported by Timmy Douglas: overflow in bit vector setter with
+;;; constant index and value.
+(loop for n-bits = 1 then (* n-bits 2)
+      for type = `(unsigned-byte ,n-bits)
+      and v-max = (1- (ash 1 n-bits))
+      while (<= n-bits sb-vm:n-word-bits)
+      do
+      (let* ((n (* 2 (1+ (- sb-vm::n-word-bits n-bits))))
+             (array1 (make-array n :element-type type))
+             (array2 (make-array n :element-type type)))
+        (dotimes (i n)
+          (dolist (v (list 0 v-max))
+            (let ((f (compile nil `(lambda (a)
+                                     (declare (type (simple-array ,type (,n)) a))
+                                     (setf (aref a ,i) ,v)))))
+              (fill array1 (- v-max v))
+              (fill array2 (- v-max v))
+              (funcall f array1)
+              (setf (aref array2 i) v)
+              (assert (every #'= array1 array2)))))))
+
+(let ((fn (compile nil '(lambda (x)
+			  (declare (type bit x))
+			  (declare (optimize speed))
+			  (let ((b (make-array 64 :element-type 'bit
+					       :initial-element 0)))
+			    (count x b))))))
+  (assert (= (funcall fn 0) 64))
+  (assert (= (funcall fn 1) 0)))
+
+(let ((fn (compile nil '(lambda (x y)
+			  (declare (type simple-bit-vector x y))
+			  (declare (optimize speed))
+			  (equal x y)))))
+  (assert (funcall 
+	   fn 
+	   (make-array 64 :element-type 'bit :initial-element 0)
+	   (make-array 64 :element-type 'bit :initial-element 0)))
+  (assert (not 
+	   (funcall 
+	    fn
+	    (make-array 64 :element-type 'bit :initial-element 0)
+	    (let ((b (make-array 64 :element-type 'bit :initial-element 0)))
+	      (setf (sbit b 63) 1)
+	      b)))))
+
+;;; MISC.535: compiler failure
+(let ((c0 #c(4196.088977268509d0 -15943.3603515625d0)))
+    (assert (not (funcall
+     (compile
+      nil
+      `(lambda (p1 p2)
+	 (declare (optimize speed (safety 1))
+		  (type (eql ,c0) p1)
+		  (type number p2))
+	 (eql (the (complex double-float) p1) p2)))
+     c0 #c(12 612/979)))))
+
+;;; reported by Lutz Euler: we shouldn't signal a compiler note for
+;;; simple-bit-vector functions.
+(handler-bind ((sb-ext:compiler-note #'error))
+  (compile nil '(lambda (x)
+		 (declare (type simple-bit-vector x))
+		 (count 1 x))))
+(handler-bind ((sb-ext:compiler-note #'error))
+  (compile nil '(lambda (x y)
+		 (declare (type simple-bit-vector x y))
+		 (equal x y))))
+
+;;; MISC.550: CAST merging in IR1 finalization caused unexpected
+;;; code transformations.
+(assert (eql (funcall
+  (compile
+   nil
+   '(lambda (p1 p2)
+     (declare (optimize (speed 3) (safety 2) (debug 3) (space 3))
+      (type atom p1)
+      (type symbol p2))
+     (or p1 (the (eql t) p2))))
+   nil t)
+  t))
+
+;;; MISC.548: type check weakening converts required type into
+;;; optional
+(assert (eql t
+  (funcall
+   (compile
+    nil
+    '(lambda (p1)
+      (declare (optimize (speed 2) (safety 1) (debug 3) (space 2)))
+      (atom (the (member f assoc-if write-line t w) p1))))
+   t)))
+
+;;; Free special bindings only apply to the body of the binding form, not
+;;; the initialization forms.
+(assert (eq :good
+	    (funcall (compile 'nil
+			      (lambda ()
+				(let ((x :bad))
+				  (declare (special x))
+				  (let ((x :good))
+				    ((lambda (&optional (y x))
+				       (declare (special x)) y)))))))))
