@@ -91,7 +91,27 @@
 	   least-positive-double-float))
 (assert (= 0.0 (scale-float 1.0 most-negative-fixnum)))
 (assert (= 0.0d0 (scale-float 1.0d0 (1- most-negative-fixnum))))
-(assert (raises-error? (scale-float 1.0 most-positive-fixnum)
-		       floating-point-overflow))
-(assert (raises-error? (scale-float 1.0d0 (1+ most-positive-fixnum))
-		       floating-point-overflow))
+#-(or darwin) ;; bug 372
+(progn
+  (assert (raises-error? (scale-float 1.0 most-positive-fixnum)
+			 floating-point-overflow))
+  (assert (raises-error? (scale-float 1.0d0 (1+ most-positive-fixnum))
+			 floating-point-overflow)))
+
+;;; bug found by jsnell when nfroyd tried to implement better LOGAND
+;;; type derivation.
+(assert (= (integer-decode-float (coerce -1756510900000000000
+                                         'single-float))
+           12780299))
+
+;;; MISC.564: no out-of-line %ATAN2 for constant folding
+(assert (typep
+  (funcall
+   (compile
+    nil
+    '(lambda (p1)
+      (declare (optimize (speed 3) (safety 2) (debug 3) (space 0))
+       (type complex p1))
+      (phase (the (eql #c(1.0d0 2.0d0)) p1))))
+   #c(1.0d0 2.0d0))
+    'double-float))
